from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
__docformat__ = 'restructuredtext'

import io
import pickle
import os
import sys
import gzip

from fipy.tools import parallelComm

__all__ = ["write", "read"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

# TODO: add test to show that round trip pickle of mesh doesn't work properly
# FIXME: pickle fails to work properly on numpy 1.1 (run gapFillMesh.py)
def write(data, filename = None, extension = '', communicator=parallelComm):
    """
    Pickle an object and write it to a file. Wrapper for
    `cPickle.dump()`.

    Test to check pickling and unpickling.

        >>> from fipy.meshes import Grid1D
        >>> old = Grid1D(nx = 2)
        >>> f, tempfile = write(old)
        >>> new = read(tempfile, f)
        >>> print(old.numberOfCells == new.numberOfCells)
        True

    Parameters
    ----------
    data
        Object to be pickled.
    filename : str
        Name of the file to place the pickled object.  If `filename` is
        `None` then a temporary file will be used and the file object and
        file name will be returned as a tuple
    extension : str
        Used if `filename` is not given.
    communicator : ~fipy.tools.comms.commWrapper.CommWrapper
        A duck-typed object with `procID` and `Nproc` attributes is sufficient
    """
    if communicator.procID == 0:
        if filename is None:
            import tempfile
            (f, _filename) =  tempfile.mkstemp(extension)
        else:
            (f, _filename) = (None, filename)
        fileStream = gzip.GzipFile(filename=_filename, mode='wb', fileobj=None)
    else:
        fileStream = open(os.devnull, mode='wb')
        (f, _filename) = (None, os.devnull)

    pickle.dump(data, fileStream, 0)
    fileStream.close()

    if filename is None:
        return (f, _filename)

def read(filename, fileobject=None, communicator=parallelComm, mesh_unmangle=False):
    """
    Read a pickled object from a file. Returns the unpickled object.
    Wrapper for `cPickle.load()`.

    Parameters
    ----------
    filename : str
        Name of the file to unpickle the object from.
    fileobject : file
        Used to remove temporary files
    communicator : ~fipy.tools.comms.commWrapper.CommWrapper
        A duck-typed object with `procID` and `Nproc` attributes is sufficient
    mesh_unmangle : bool
        Whether to correct improper pickling of non-uniform meshes (ticket:243)
    """
    if communicator.procID == 0:
        fileStream = gzip.GzipFile(filename=filename, mode='r', fileobj=None)
        data = fileStream.read()
        fileStream.close()
        if fileobject is not None:
            os.close(fileobject)
            os.remove(filename)
    else:
        data = None

    if communicator.Nproc > 1:
        data = communicator.bcast(data, root=0)

    f = io.BytesIO(data)

    unpickler = pickle.Unpickler(f)

    if mesh_unmangle:
        def find_class(module, name):
            __import__(module)
            mod = sys.modules[module]
            klass = getattr(mod, name)

            from fipy import meshes
            import types

            if isinstance(klass, type) and issubclass(klass, meshes.mesh.Mesh):
                class UnmangledMesh(klass):
                    def __setstate__(self, dict):
                        if ('cellFaceIDs' in dict
                            and 'faceVertexIDs' in dict):

                            dict = dict.copy()
                            for key in ('cellFaceIDs', 'faceVertexIDs'):
                                arr = dict[key]
                                arr.data[:] = arr.transpose().flatten().reshape(arr.shape)

                        klass.__setstate__(self, dict)

                return UnmangledMesh
            else:
                return klass

        unpickler.find_global = find_class

    return unpickler.load()

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()




from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

from fipy.solvers.solver import Solver
from fipy.matrices.pysparseMatrix import _PysparseMeshMatrix

class _PysparseMatrixSolver(Solver):

    """
    A class consolidating methods for solver packages which use
    `_PysparseMeshMatrix` for their matrix class.

    Subclasses have a `_solve_` method, which is called by `_solve`. Typically,
    `_solve_` returns the new value of `self.var` to `_solve` and solve sets the
    var accordingly.

    A solution function `solveFnc`, usually of the form `solve(A, x, b)`, is
    implemented in most leaf-node child classes.

    .. attention:: This class is abstract. Always create one of its subclasses.
    """

    solveFnc = None

    @property
    def _matrixClass(self):
        return _PysparseMeshMatrix

    def _solve(self):
        """
        Call `_solve_` for the new value of `self.var`.

        In certain cases, `_solve_` won't return anything, e.g.
        `fipy.solvers.pysparse.linearLUSolver`. In these cases, we preserve the
        value of `self.var.numericValue`.
        """

        if self.var.mesh.communicator.Nproc > 1:
            raise Exception("%ss cannot be used with multiple processors" \
                            % self.__class__)

        array = self.var.numericValue
        newArr = self._solve_(self.matrix, array, self.RHSvector)

        if newArr is not None:
            array = newArr

        factor = self.var.unit.factor

        if factor != 1:
            array /= self.var.unit.factor

        self.var[:] = array



"""setuptools command to prepare FiPy for release"""
from __future__ import unicode_literals

from distutils.core import Command
import glob
import os
import shutil

from setuptools.sandbox import run_setup
from future.utils import text_to_native_str

from ._nativize import nativize_all

__all__ = [text_to_native_str("release")]


class release(Command):
    """Prepare FiPy for release

    Generates tarball and excutable Windows installer"""

    description = "Prepare the FiPy release artifacts"

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    user_options = [('unix', None, "create a tarball source distribution"),
                    ('windows', None, "create an executable installer for MS Windows"),
                    ('all', None, "create unix and Windows distributions"),
                   ]
    user_options = [nativize_all(u) for u in user_options]

    def initialize_options(self):
        self.unix = 0
        self.windows = 0
        self.all = 0

    def finalize_options(self):
        if self.all:
            self.unix = 1
            self.windows = 1

    def _remove_manifest(self):
        """Remove MANIFEST file

        probably no longer needed, MANIFEST was ancient history?"""

        try:
            os.remove("MANIFEST")
        except OSError as _:
            pass

    def _build_unix_distribution(self):
        """Create Unix source distribution"""

        self._remove_manifest()
        shutil.copyfile("MANIFEST-UNIX.in", "MANIFEST.in")
        run_setup("setup.py", ["sdist"])
        os.remove("MANIFEST.in")

    def _build_windows_distribution(self):
        """Create Windows source distribution

        Contains executable installer and examples"""

        import versioneer

        version = versioneer.get_version()

        self._remove_manifest()
        run_setup("setup.py", ["bdist_wininst"])

        self._remove_manifest()

        shutil.copyfile("MANIFEST-WINDOWS.in", "MANIFEST.in")
        run_setup("setup.py", [text_to_native_str(s) for s in ["sdist", "--dist-dir=dist-windows", "--formats=zip"]])
        shutil.move(
            os.path.join("dist-windows", "FiPy-{}.zip".format(version)),
            os.path.join("dist", "FiPy-{}.win32.zip".format(version)),
        )
        os.rmdir("dist-windows")
        os.remove("MANIFEST.in")

    def run(self):
        if self.unix:
            self._build_unix_distribution()
        if self.windows:
            self._build_windows_distribution()



from __future__ import division
from builtins import zip
from builtins import range
__docformat__ = 'restructuredtext'

__all__ = []

from petsc4py import PETSc

from fipy.tools import numerix
from fipy.matrices.sparseMatrix import _SparseMatrix

class _PETScMatrix(_SparseMatrix):
    
    def __init__(self, matrix):
        """Creates a wrapper for a PETSc matrix

        Allows basic python operations __add__, __sub__ etc.
        Facilitate matrix populating in an easy way.
        
        :Parameters:
          - `matrix`: The starting `PETSc.Mat` 
        """
        self.matrix = matrix
        
    def getCoupledClass(self):
        return _CoupledPETScMeshMatrix
    
    def copy(self):
        return _PETScMatrix(matrix=self.matrix.copy())
        
    def __getitem__(self, index):
        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()
        m = self.matrix[index]
        if numerix.shape(m) == ():
            return m
        else:
            return _PETScMatrix(matrix=m)
    
    def __str__(self):
        self.matrix.assemble()

        return _SparseMatrix.__str__(self)

    def __iadd__(self, other):
        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()
        if other != 0:
            other.matrix.assemblyBegin()
            other.matrix.assemblyEnd()
            self.matrix = self.matrix + other.matrix
        return self

    def __add__(self, other):
        """
        Add two sparse matrices
        
            >>> L = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=3)
            >>> L.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
            >>> print(L + _PETScIdentityMatrix(size=3))
             1.000000  10.000000   3.000000  
                ---     4.141593      ---    
             2.500000      ---     1.000000  
             
            >>> print(L + 0)
                ---    10.000000   3.000000  
                ---     3.141593      ---    
             2.500000      ---        ---    
            
            >>> print(L + 3)
            Traceback (most recent call last):
            ...
            AttributeError: 'int' object has no attribute 'matrix'
        """

        if other == 0:
            return self
        else:
            self.matrix.assemblyBegin()
            self.matrix.assemblyEnd()
            other.matrix.assemblyBegin()
            other.matrix.assemblyEnd()
            return _PETScMatrix(matrix=self.matrix + other.matrix)
        
    __radd__ = __add__
    
    def __sub__(self, other):

        if other == 0:
            return self
        else:
            self.matrix.assemblyBegin()
            self.matrix.assemblyEnd()
            other.matrix.assemblyBegin()
            other.matrix.assemblyEnd()
            return _PETScMatrix(matrix=self.matrix - other.matrix)

    def __rsub__(self, other):
        return -self + other
    
    def __isub__(self, other):
        if other != 0:
            self.matrix.assemblyBegin()
            self.matrix.assemblyEnd()
            other.matrix.assemblyBegin()
            other.matrix.assemblyEnd()
            self.matrix = self.matrix - other.matrix
        return self

    def __mul__(self, other):
        """
        Multiply a sparse matrix by another sparse matrix
        
            >>> L1 = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=2)
            >>> L1.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
            >>> L2 = _PETScIdentityMatrix(size=3, bandwidth=3)
            >>> L2.put([4.38], [2], [1])
            >>> L2.put([4.38,12357.2,1.1], [2,1,0], [1,0,2])
            
            >>> tmp = numerix.array(((1.23572000e+05, 2.31400000e+01, 3.00000000e+00),
            ...                      (3.88212887e+04, 3.14159265e+00, 0.00000000e+00),
            ...                      (2.50000000e+00, 0.00000000e+00, 2.75000000e+00)))

            >>> numerix.allclose((L1 * L2).numpyArray, tmp)
            1

        or a sparse matrix by a vector

            >>> tmp = numerix.array((29., 6.28318531, 2.5))       
            >>> numerix.allclose(L1 * numerix.array((1,2,3),'d'), tmp)
            1
            
        or a vector by a sparse matrix

            >>> tmp = numerix.array((7.5, 16.28318531,  3.))  
            >>> numerix.allclose(numerix.array((1,2,3),'d') * L1, tmp) ## The multiplication is broken. Numpy is calling __rmul__ for every element instead of with  the whole array.
            1

            
        """
        N = self._shape[1]

        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()

        if isinstance(other, _PETScMatrix):
            other.matrix.assemblyBegin()
            other.matrix.assemblyEnd()
            copy = self.copy()
            copy.matrix = self.matrix.matMult(other.matrix)
            return copy
        elif isinstance(other, PETSc.Vec):
            y = other.duplicate()
            self.matrix.mult(other, y)
            return y
        else:
            shape = numerix.shape(other)
            if shape == ():
                result = self.copy()
                result.matrix = self.matrix * other
                return result
            elif shape == (N,):
                y = PETSc.Vec().createWithArray(other, comm=self.matrix.comm)
                result = y.duplicate()
                self.matrix.mult(y, result)
                return result
            else:
                raise TypeError
            
    def __rmul__(self, other):
        if type(numerix.ones(1, 'l')) == type(other):
            N = self._shape[1]
            x = PETSc.Vec().createMPI(N, comm=self.matrix.comm)
            y = x.duplicate()
            x[:] = other
            self.matrix.multTranspose(x, y)
            return numerix.asarray(y)
        else:
            return self * other
            
    @property
    def _shape(self):
        return self.matrix.sizes[0][0], self.matrix.sizes[1][1]

    @property
    def _range(self):
        return list(range(self._shape[1])), list(range(self._shape[0]))
        
    def put(self, vector, id1, id2):
        """
        Put elements of `vector` at positions of the matrix corresponding to (`id1`, `id2`)
        
            >>> L = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=2)
            >>> L.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
            >>> print(L)
                ---    10.000000   3.000000  
                ---     3.141593      ---    
             2.500000      ---        ---    
        """
        self.matrix.setValuesCSR(*self._ijv2csr(id2, id1, vector))
            
    def _ijv2csr(self, i, j, v):
        """Convert arrays of matrix indices and values into CSR format
        
        see: http://netlib.org/linalg/html_templates/node91.html#SECTION00931100000000000000

        .. note::
           petsc4py only understands CSR formatted matrices (setValuesCSR and
           setValuesIJV both inexplicably call the same underlying routine).
        
        Parameters
        ----------
        i : array_like
            column indices
        j : array_like
            row indices
        v : array_like
            non-zero values
            
        Returns
        -------
        row_ptr : array_like
            locations in the val vector that start a row, 
            terminated with len(val) + 1
        cols : array_like
            column indices
        val : array_like
            non-zero values
        """
#         from fipy.tools.debug import PRINT
#         PRINT("i:", i)
#         PRINT("j:", j)
#         PRINT("v:", v)
        
        i = numerix.asarray(i)
        j = numerix.asarray(j)
        v = numerix.asarray(v)
        start_row, end_row = self.matrix.getOwnershipRange()
#         PRINT("start_row:", start_row)
#         PRINT("end_row:", end_row)

        ix = numerix.lexsort([i, j])
        ix = ix[(j[ix] >= start_row) & (j[ix] < end_row)]
        cols = i[ix]
        row_ptr = numerix.searchsorted(j[ix], 
#                                        numerix.arange(0, self._shape[1]+1))
                                       numerix.arange(start_row, end_row + 1))
        vals = v[ix]
        
#         PRINT("i[ix]:", i[ix])
#         PRINT("j[ix]:", j[ix])
#         PRINT("v[ix]:", v[ix])

        
        
#         PRINT("size:", self._shape)
#         PRINT("shape:", self._shape[0])
#         
#         PRINT("row_ptr:", row_ptr)
#         PRINT("cols:", cols)
#         PRINT("vals:", vals)
#         
#         PRINT("size:", self.matrix.sizes)
#         PRINT("owns-rows:", self.matrix.getOwnershipRange())
#         PRINT("owns-cols:", self.matrix.getOwnershipRangesColumn())
        
        # note: PETSc (at least via pip) only seems to handle 32 bit addressing
        return row_ptr.astype('int32'), cols.astype('int32'), vals

    def putDiagonal(self, vector):
        """
        Put elements of `vector` along diagonal of matrix
        
            >>> L = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=1)
            >>> L.putDiagonal([3.,10.,numerix.pi])
            >>> print(L)
             3.000000      ---        ---    
                ---    10.000000      ---    
                ---        ---     3.141593  
            >>> L.putDiagonal([10.,3.])
            >>> print(L)
            10.000000      ---        ---    
                ---     3.000000      ---    
                ---        ---     3.141593  
        """
        if type(vector) in [type(1), type(1.)]:
            ids = numerix.arange(self._shape[0])
            tmp = numerix.zeros((self._shape[0],), 'd')
            tmp[:] = vector
            self.put(tmp, ids, ids)
        else:
            ids = numerix.arange(len(vector))
            self.put(vector, ids, ids)

    def take(self, id1, id2):
        # FIXME: This is a terrible implementation
        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()
        vector = [self.matrix[i, j] for i, j in zip(id1, id2)]
        vector = numerix.array(vector, 'd')
        return vector

    def takeDiagonal(self):
        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()

        return self.matrix.getDiagonal().array

    def addAt(self, vector, id1, id2):
        """
        Add elements of `vector` to the positions in the matrix corresponding to (`id1`,`id2`)
        
            >>> L = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=3)
            >>> L.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
            >>> L.addAt([1.73,2.2,8.4,3.9,1.23], [1,2,0,0,1], [2,2,0,0,2])
            >>> print(L)
            12.300000  10.000000   3.000000  
                ---     3.141593   2.960000  
             2.500000      ---     2.200000  
        """
        self.matrix.setValuesCSR(*self._ijv2csr(id2, id1, vector),
                                 addv=True)

    def addAtDiagonal(self, vector):
        if type(vector) in [type(1), type(1.)]:
            ids = numerix.arange(self._shape[0])
            tmp = numerix.zeros((self._shape[0],), 'd')
            tmp[:] = vector
            self.addAt(tmp, ids, ids)
        else:
            ids = numerix.arange(len(vector))
            self.addAt(vector, ids, ids)
            
    def _petsc2app(self, ids):
        return ids

    @property
    def _scipy_csr(self):
        """Return the PETSc-ordered CSR matrix
        """
        from scipy import sparse
        mpi4pycomm = self.matrix.comm.tompi4py()

        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()

        indptr, indices, data = self.matrix.getValuesCSR()

        # getValuesCSR() returns entries local to node
        # with node-relative indptr.
        # sparse.csr_matrix() requires all elements for construction
        # and global indptr

        offset = numerix.cumsum([0] + mpi4pycomm.allgather(len(data)))
        offset = mpi4pycomm.scatter(offset[:-1])

        indices = numerix.concatenate(mpi4pycomm.allgather(indices))
        data = numerix.concatenate(mpi4pycomm.allgather(data))

        # strip local end markers and append global end marker
        indptr = mpi4pycomm.allgather(indptr[:-1] + offset) + [[len(data)]]
        indptr = numerix.concatenate(indptr)

        (rows, globalRows), (cols, globalCols) = self.matrix.getSizes()

        return sparse.csr_matrix((data, indices, indptr),
                                 shape=(globalRows, globalCols))

    @property
    def _scipy_coo(self):
        """Return the application-ordered COO matrix
        """
        from scipy import sparse

        coo = self._scipy_csr.tocoo()
        return sparse.coo_matrix((coo.data,
                                  (self._petsc2app(coo.row),
                                   self._petsc2app(coo.col))),
                                 shape=coo.shape)

    @property
    def numpyArray(self):
        # self.matrix.getDenseArray() raises
        # [0] MatDenseGetArray() line 1782 in .../src/mat/impls/dense/seq/dense.c
        # [0] No support for this operation for this object type
        # [0] Cannot locate function MatDenseGetArray_C in object
        return self._scipy_coo.toarray()
                
    def matvec(self, x):
        """This method is required for scipy solvers.
        """
        return self * x

    def exportMmf(self, filename):
        """Exports the matrix to a Matrix Market file of the given `filename`.
        """
        from scipy.io import mmio

        mmio.mmwrite(filename, self._scipy_coo)
    
class _PETScMatrixFromShape(_PETScMatrix):
    
    def __init__(self, rows, cols, bandwidth=0, sizeHint=None, matrix=None, comm=PETSc.COMM_SELF):
        """Instantiates and wraps a PETSc `Mat` matrix

        :Parameters:
          - `rows`: The number of matrix rows
          - `cols`: The number of matrix columns
          - `bandwidth`: The proposed band width of the matrix.
          - `sizeHint`: estimate of the number of non-zeros
          - `matrix`: pre-assembled `ll_mat` to use for storage
          - `comm`: communicator
          
        """
        bandwidth = bandwidth 
        if (bandwidth == 0) and (sizeHint is not None):
            bandwidth = sizeHint // max(rows, cols)
        if matrix is None:
            matrix = PETSc.Mat()
            matrix.create(comm)
            # rows are owned per process
            # cols are owned by everyone
            matrix.setSizes([[rows, None], [cols, None]])
            matrix.setType('aij') # sparse
            matrix.setPreallocationNNZ(None) # FIXME: ??? #bandwidth)
                
        _PETScMatrix.__init__(self, matrix=matrix)

class _PETScMeshMatrix(_PETScMatrixFromShape):
    def __init__(self, mesh, bandwidth=0, sizeHint=None, matrix=None, numberOfVariables=1, numberOfEquations=1):

        """Creates a `_PETScMatrixFromShape` associated with a `Mesh`. Allows for different number of equations and/or variables

        :Parameters:
          - `mesh`: The `Mesh` to assemble the matrix for.
          - `bandwidth`: The proposed band width of the matrix.
          - `sizeHint`: estimate of the number of non-zeros
          - `matrix`: pre-assembled `ll_mat` to use for storage
          - `numberOfVariables`: The columns of the matrix is determined by `numberOfVariables * self.mesh.numberOfCells`.
          - `numberOfEquations`: The rows of the matrix is determined by `numberOfEquations * self.mesh.numberOfCells`.
        """
        self.mesh = mesh
        self.numberOfVariables = numberOfVariables
        self.numberOfEquations = numberOfEquations

        _PETScMatrixFromShape.__init__(self, 
                                       rows=numberOfEquations * len(self.mesh._localNonOverlappingCellIDs), 
                                       cols=numberOfVariables * len(self.mesh._localNonOverlappingCellIDs), # self.mesh.globalNumberOfCells, # , # 
                                       bandwidth=bandwidth, 
                                       sizeHint=sizeHint, 
                                       matrix=matrix,
                                       comm=mesh.communicator.petsc4py_comm)
                                       
    
    @property
    def _ao(self):
        """Application Ordering to relate FiPy matrix rows to PETSc matrix rows
        
        FiPy naturally blocks matrix rows, one set of Equations (or Variables) at a time.
        PETSc requires that all rows pertaining to a particular MPI node be contiguous.
        This PETSc `AO` (Application Ordering) object converts between them.
        
        Only needed for FiPy to PETSc. We can efficiently slice from PETSc to
        FiPy, but PETSc requires us to know the row IDs. 
        """ 
        if not hasattr(self, "_ao_"): 
            comm = self.mesh.communicator
            
            from mpi4py import MPI

            fipyIDs = self._globalNonOverlappingColIDs
            N = len(fipyIDs)

            count = numerix.zeros((comm.Nproc,), dtype=int)
            count[comm.procID] = N
            comm.mpi4py_comm.Allreduce(sendbuf=MPI.IN_PLACE, recvbuf=count, op=MPI.MAX)
            
            petscIDs = numerix.arange(N) + numerix.sum(count[:comm.procID])
            
            self._ao_ = PETSc.AO().createBasic(petsc=petscIDs.astype('int32'), 
                                               app=fipyIDs.astype('int32'), 
                                               comm=comm.petsc4py_comm)
        return self._ao_
        
    def _petsc2app(self, ids):
        return self._ao.petsc2app(ids)

    def _cellIDsToGlobalRowIDs(self, IDs):
         N = len(IDs)
         M = self.numberOfEquations
         return (numerix.vstack([IDs] * M) + numerix.indices((M,N))[0] * self.mesh.globalNumberOfCells).flatten()

    def _cellIDsToGlobalColIDs(self, IDs):
         N = len(IDs)
         M = self.numberOfVariables
         return (numerix.vstack([IDs] * M) + numerix.indices((M,N))[0] * self.mesh.globalNumberOfCells).flatten()

    def _cellIDsToLocalRowIDs(self, IDs):
         M = self.numberOfEquations
         N = len(IDs)
         return (numerix.vstack([IDs] * M) + numerix.indices((M,N))[0] * self.mesh.numberOfCells).flatten()

    def _cellIDsToLocalColIDs(self, IDs):
         M = self.numberOfVariables
         N = len(IDs)
         return (numerix.vstack([IDs] * M) + numerix.indices((M,N))[0] * self.mesh.numberOfCells).flatten()

    @property
    def _globalNonOverlappingRowIDs(self):
        return self._cellIDsToGlobalRowIDs(self.mesh._globalNonOverlappingCellIDs)

    @property
    def _globalNonOverlappingColIDs(self):
        return self._cellIDsToGlobalColIDs(self.mesh._globalNonOverlappingCellIDs)

    @property
    def _globalOverlappingRowIDs(self):
        return self._cellIDsToGlobalRowIDs(self.mesh._globalOverlappingCellIDs)

    @property
    def _globalCommonColIDs(self):
        return list(range(0, self.numberOfVariables, self.mesh.globalNumberOfCells))
                     
    @property
    def _globalOverlappingColIDs(self):
        return self._cellIDsToGlobalColIDs(self.mesh._globalOverlappingCellIDs)

    @property
    def _localNonOverlappingRowIDs(self):
        return self._cellIDsToLocalRowIDs(self.mesh._localNonOverlappingCellIDs)

    @property
    def _localOverlappingColIDs(self):
        return self._cellIDsToLocalColIDs(self.mesh._localOverlappingCellIDs)

    @property
    def _localNonOverlappingColIDs(self):
        return self._cellIDsToLocalColIDs(self.mesh._localNonOverlappingCellIDs)

    def copy(self):
        tmp = _PETScMatrixFromShape.copy(self)
        copy = self.__class__(mesh=self.mesh) # FIXME: ??? , bandwidth=self.bandwidth)
        copy.matrix = tmp.matrix
        return copy

    def _getStencil(self, id1, id2):
        id1 = self._globalOverlappingRowIDs[id1]
        id2 = self._globalOverlappingColIDs[id2]
            
        mask = numerix.in1d(id1, self._globalNonOverlappingRowIDs) 
        id1 = self._ao.app2petsc(id1[mask].astype('int32'))
        id2 = self._ao.app2petsc(id2[mask].astype('int32'))
        
        return id1, id2, mask

    def _globalNonOverlapping(self, vector, id1, id2):
        """Transforms and subsets local overlapping values and coordinates to global non-overlapping
        
        :Parameters:
          - `vector`: The overlapping values to insert.
          - `id1`: The local overlapping row indices.
          - `id2`: The local overlapping column indices.
          
        :Returns: 
          Tuple of (non-overlapping vector, 
                    global non-overlapping row indices, 
                    global non-overlapping column indices)
        """
        id1, id2, mask = self._getStencil(id1, id2)
        vector = vector[mask]
        return (vector, id1, id2)

    def put(self, vector, id1, id2):
        vector, id1, id2 = self._globalNonOverlapping(vector, id1, id2)
        _PETScMatrixFromShape.put(self, vector=vector, id1=id1, id2=id2)

    def addAt(self, vector, id1, id2):
        vector, id1, id2 = self._globalNonOverlapping(vector, id1, id2)
        _PETScMatrixFromShape.addAt(self, vector=vector, id1=id1, id2=id2)
    
    @property
    def _bodies(self):
        if not hasattr(self, "_bodies_"):
            self._bodies_ = numerix.in1d(self.mesh._globalOverlappingCellIDs, 
                                         self.mesh._globalNonOverlappingCellIDs)
        return self._bodies_
        
    @property
    def _ghosts(self):
        if not hasattr(self, "_ghosts_"):
            self._ghosts_ = self.mesh._globalOverlappingCellIDs[~self._bodies]
            self._ghosts_ = self._cellIDsToGlobalRowIDs(self._ghosts_)
            self._ghosts_ = self._ao.app2petsc(self._ghosts_.astype('int32'))
            
        return self._ghosts_
        
    def _fipy2petscGhost(self, var):
        """Convert a FiPy Variable to a PETSc `GhostVec`
        
        Moves the ghosts to the end, as necessary. 
        `var` may be coupled/vector and so moving the ghosts is a bit subtle.
        
        Given a (2x4) vector variable `vij`
        
        ```
        v00  v01 (v02)        processor 0
        v10  v11 (v12)
        
            (v01) v02 v03     processor 1
            (v11) v12 v13
        ```
            
        where i is the vector index and j is the global index.
        Elements in () are ghosted
        
        We end up with the `GhostVec`
        
        ```
        v00 v01 v10 v11 (v02) (v12)   [4, 6]  processor 0
        v02 v03 v12 v13 (v01) (v11)   [1, 3]  processor 1
        ```
        
        where the [a, b] are the global ghost indices
        """
        corporeal = numerix.asarray(var[..., self._bodies]).ravel()
        incorporeal = numerix.asarray(var[..., ~self._bodies]).ravel()
        array = numerix.concatenate([corporeal, incorporeal])

        comm = self.mesh.communicator.petsc4py_comm
        vec = PETSc.Vec().createGhostWithArray(ghosts=self._ghosts.astype('int32'),
                                               array=array,
                                               comm=comm)
                                               
        return vec

        
    def _petsc2fipyGhost(self, vec):
        """Convert a PETSc `GhostVec` to a FiPy Variable (form)
        
        Moves the ghosts from the end, as necessary. 
        The return Variable may be coupled/vector and so moving the ghosts
        is a bit subtle.
        
        Given an 8-element `GhostVec` `vj`
        
        ```
        v0 v1 v2 v3 (v4) (v6)   [4, 6]  processor 0
        v4 v5 v6 v7 (v1) (v3)   [1, 3]  processor 1
        ```
        
        where j is the global index and the `[a, b]` are the global ghost
        indices. Elements in () are ghosted

        We end up with the (2x4) FiPy Variable
        
        ```
        v0  v1 (v4)        processor 0
        v2  v3 (v6)
        
           (v1) v4 v4      processor 1
           (v3) v6 v7
        ```
        """
        N = len(self.mesh._globalOverlappingCellIDs)
        M = self.numberOfEquations
        var = numerix.empty((M, N))
        bodies = numerix.array(vec)
        if M > 1:
            bodies = numerix.reshape(bodies, (M, -1))
        var[..., self._bodies] = bodies
        vec.ghostUpdate()
        with vec.localForm() as lf:
            if len(self._ghosts) > 0:
                ids = numerix.arange(-len(self._ghosts), 0)
                ghosts = numerix.reshape(numerix.array(lf)[ids], (M, -1))
                var[..., ~self._bodies] = ghosts

        return var.flatten()

    def __mul__(self, other):
        """Multiply a sparse matrix by another sparse matrix
        
            >>> L1 = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=2)
            >>> L1.put([3.,10.,numerix.pi,2.5], [0,0,1,2], [2,1,1,0])
            >>> L2 = _PETScIdentityMatrix(size=3, bandwidth=3)
            >>> L2.put([4.38], [2], [1])
            >>> L2.put([4.38,12357.2,1.1], [2,1,0], [1,0,2])
            
            >>> tmp = numerix.array(((1.23572000e+05, 2.31400000e+01, 3.00000000e+00),
            ...                      (3.88212887e+04, 3.14159265e+00, 0.00000000e+00),
            ...                      (2.50000000e+00, 0.00000000e+00, 2.75000000e+00)))

            >>> numerix.allclose((L1 * L2).numpyArray, tmp)
            1

        or a sparse matrix by a vector

            >>> tmp = numerix.array((29., 6.28318531, 2.5))       
            >>> numerix.allclose(L1 * numerix.array((1,2,3),'d'), tmp)
            1
            
        or a vector by a sparse matrix

            >>> tmp = numerix.array((7.5, 16.28318531,  3.))  
            >>> numerix.allclose(numerix.array((1,2,3),'d') * L1, tmp) ## The multiplication is broken. Numpy is calling __rmul__ for every element instead of with  the whole array.
            1

            
        """
        N = self._shape[1]

        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()
        
        if isinstance(other, (_PETScMatrix, PETSc.Vec)):
            return _PETScMatrixFromShape.__mul__(self, other=other)
        else:
            shape = numerix.shape(other)

            if shape == ():
                result = self.copy()
                result.matrix = self.matrix * other
                return result
            else:
                x = other[self._localNonOverlappingColIDs]
                x = PETSc.Vec().createWithArray(x, comm=self.matrix.comm)

                y = PETSc.Vec().createGhost(ghosts=self._ghosts.astype('int32'),
                                            size=(len(self._localNonOverlappingColIDs), None),
                                            comm=self.matrix.comm)
                self.matrix.mult(x, y)
                return self._petsc2fipyGhost(vec=y)

    def takeDiagonal(self):
        self.matrix.assemblyBegin()
        self.matrix.assemblyEnd()

        y = PETSc.Vec().createGhost(ghosts=self._ghosts.astype('int32'),
                                    size=(len(self._localNonOverlappingColIDs), None),
                                    comm=self.matrix.comm)
        self.matrix.getDiagonal(result=y)
        return self._petsc2fipyGhost(vec=y)

    def flush(self):
        """Deletes the copy of the PETSc matrix held.
        """
    
        if not getattr(self, 'cache', False):
            del self.matrix

    def _test(self):
        """Tests
        
        >>> m = _PETScMatrixFromShape(rows=3, cols=3, bandwidth=1)
        >>> m.addAt((1., 0., 2.), (0, 2, 1), (1, 2, 0))
        >>> m.matrix.assemblyBegin()
        >>> m.matrix.assemblyEnd()
        
        # FIXME: are these names even right? is this a good test?
        
        >>> col, row, val = m.matrix.getValuesCSR()
        >>> print(numerix.allequal(col, [0, 1, 2, 3]))
        True
        >>> print(numerix.allequal(row, [1, 0, 2]))
        True
        >>> print(numerix.allclose(val, [1., 2., 0.]))
        True
        """
        pass
        
class _PETScIdentityMatrix(_PETScMatrixFromShape):
    """Represents a sparse identity matrix for pysparse.
    """
    def __init__(self, size, bandwidth=1, comm=PETSc.COMM_SELF):
        """Create a sparse matrix with `1` in the diagonal
        
            >>> print(_PETScIdentityMatrix(size=3))
             1.000000      ---        ---    
                ---     1.000000      ---    
                ---        ---     1.000000  
        """
        _PETScMatrixFromShape.__init__(self, rows=size, cols=size, bandwidth=bandwidth, comm=comm)
        ids = numerix.arange(size)
        self.put(numerix.ones(size, 'd'), ids, ids)
        
class _PETScIdentityMeshMatrix(_PETScIdentityMatrix):
    def __init__(self, mesh, bandwidth=1):
        """Create a sparse matrix associated with a `Mesh` with `1` in the diagonal
        
            >>> from fipy import Grid1D
            >>> from fipy.tools import serialComm
            >>> mesh = Grid1D(nx=3, communicator=serialComm)
            >>> print(_PETScIdentityMeshMatrix(mesh=mesh))
             1.000000      ---        ---    
                ---     1.000000      ---    
                ---        ---     1.000000  
        """
        _PETScIdentityMatrix.__init__(self, size=mesh.numberOfCells, bandwidth=bandwidth, 
                                      comm=mesh.communicator.petsc4py_comm)

def _test(): 
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()
    
if __name__ == "__main__": 
    _test()



from __future__ import division
from __future__ import unicode_literals
from builtins import range
__docformat__ = 'restructuredtext'

import os

from scipy.sparse.linalg import splu

from fipy.solvers.scipy.scipySolver import _ScipySolver
from fipy.tools import numerix

__all__ = ["LinearLUSolver"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class LinearLUSolver(_ScipySolver):
    """
    The `LinearLUSolver` solves a linear system of equations using
    LU-factorization.  The `LinearLUSolver` is a wrapper class for the
    the Scipy `scipy.sparse.linalg.splu` module.
    """

    def _solve_(self, L, x, b):
        diag = L.takeDiagonal()
        maxdiag = max(numerix.absolute(diag))

        L = L * (1 / maxdiag)
        b = b * (1 / maxdiag)

        LU = splu(L.matrix.asformat("csc"), diag_pivot_thresh=1.,
                                            relax=1,
                                            panel_size=10,
                                            permc_spec=3)

        error0 = numerix.sqrt(numerix.sum((L * x - b)**2))

        for iteration in range(min(self.iterations, 10)):
            errorVector = L * x - b

            if (numerix.sqrt(numerix.sum(errorVector**2)) / error0)  <= self.tolerance:
                break

            xError = LU.solve(errorVector)
            x[:] = x - xError

        if 'FIPY_VERBOSE_SOLVER' in os.environ:
            from fipy.tools.debug import PRINT
            PRINT('iterations: %d / %d' % (iteration+1, self.iterations))
            PRINT('residual:', numerix.sqrt(numerix.sum(errorVector**2)))

        return x



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

import os

from fipy.solvers.scipy.scipySolver import _ScipySolver

class _ScipyKrylovSolver(_ScipySolver):
    """
    The base `ScipyKrylovSolver` class.

    .. attention:: This class is abstract. Always create one of its subclasses.
    """

    def _solve_(self, L, x, b):
        A = L.matrix
        if self.preconditioner is None:
            M = None
        else:
            M = self.preconditioner._applyToMatrix(A)

        x, info = self.solveFnc(A, b, x,
                                tol=self.tolerance,
                                maxiter=self.iterations,
                                M=M,
                                atol='legacy')

        if 'FIPY_VERBOSE_SOLVER' in os.environ:
            if info < 0:
                PRINT('failure', self._warningList[info].__class__.__name__)

        return x



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.viewers.matplotlibViewer.matplotlib2DViewer import AbstractMatplotlib2DViewer

__all__ = ["Matplotlib2DGridViewer"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class Matplotlib2DGridViewer(AbstractMatplotlib2DViewer):
    """
    Displays an image plot of a 2D `CellVariable` object using Matplotlib_.

    .. _Matplotlib: http://matplotlib.sourceforge.net/
    """

    __doc__ += AbstractMatplotlib2DViewer._test2D(viewer="Matplotlib2DGridViewer")

    def __init__(self, vars, title=None, limits={}, cmap=None, colorbar='vertical', axes=None, figaspect='auto', **kwlimits):
        """Creates a `Matplotlib2DGridViewer`.

        Parameters
        ----------
        vars : ~fipy.variables.cellVariable.CellVariable
            the `Variable` to display.
        title : str, optional
            displayed at the top of the `Viewer` window
        limits : dict, optional
            a (deprecated) alternative to limit keyword arguments
        cmap : ~matplotlib.colors.Colormap, optional
            the :class:`~matplotlib.colors.Colormap`.
            Defaults to `matplotlib.cm.jet`
        float xmin, xmax, ymin, ymax, datamin, datamax : float, optional
            displayed range of data. Any limit set to
            a (default) value of `None` will autoscale.
        colorbar : bool, optional
            plot a color bar in specified orientation if not `None`
        axes : ~matplotlib.axes.Axes, optional
            if not `None`, `vars` will be plotted into this Matplotlib `Axes` object
        figaspect : float, optional
            desired aspect ratio of figure. If arg is a number, use that aspect
            ratio. If arg is `auto`, the aspect ratio will be determined from
            the Variable's mesh.
        """
        kwlimits.update(limits)
        AbstractMatplotlib2DViewer.__init__(self, vars=vars, title=title,
                                            cmap=cmap, colorbar=colorbar, axes=axes, figaspect=figaspect,
                                            **kwlimits)

        self.axes.set_xlim(xmin=self._getLimit('xmin'),
                           xmax=self._getLimit('xmax'))

        self.axes.set_ylim(ymin=self._getLimit('ymin'),
                           ymax=self._getLimit('ymax'))

        if title is None:
            self.axes.set_title(self.vars[0].name)

    def _getLimit(self, key, default=None):
        limit = AbstractMatplotlib2DViewer._getLimit(self, key, default=default)
        if limit is None:
            X, Y = self.vars[0].mesh.faceCenters
            if 'xmin' in key:
                limit = float(min(X))
            elif 'ymin' in key:
                limit = float(min(Y))
            if 'xmax' in key:
                limit = float(max(X))
            elif 'ymax' in key:
                limit = float(max(Y))
        return limit

    def _getSuitableVars(self, vars):
        from fipy.meshes.uniformGrid2D import UniformGrid2D
        from fipy.variables.cellVariable import CellVariable
        vars = [var for var in AbstractMatplotlib2DViewer._getSuitableVars(self, vars) \
          if (isinstance(var.mesh, UniformGrid2D) and isinstance(var, CellVariable)
              and var.rank == 0)]
        if len(vars) == 0:
            from fipy.viewers import MeshDimensionError
            raise MeshDimensionError("Matplotlib2DGridViewer can only display a rank-0 CellVariable with a UniformGrid2D mesh")
        # this viewer can only display one variable
        return [vars[0]]

    def _make_mappable(self):
        xmin, ymin = self.vars[0].mesh.extents['min']
        xmax, ymax = self.vars[0].mesh.extents['max']

        self.image = self.axes.imshow(self._data,
                                      extent=(xmin, xmax, ymin, ymax),
                                      norm=self.norm,
                                      cmap=self.cmap)
        return self.image

    @property
    def _data(self):
        from fipy.tools.numerix import array, reshape
        return reshape(array(self.vars[0]), self.vars[0].mesh.shape[::-1])[::-1]

    def _plot(self):
        super(Matplotlib2DGridViewer, self)._plot()
        self.image.set_data(self._data)

def _test():
    from fipy.viewers.viewer import _test2D
    _test2D(Matplotlib2DGridViewer)

if __name__ == "__main__":
    import fipy.tests.doctestPlus
    fipy.tests.doctestPlus.execButNoTest()



"""
:term:`FiPy` is an object oriented, partial differential equation (PDE) solver,
written in :term:`Python`, based on a standard finite volume (FV) approach. The
framework has been developed in the Materials Science and Engineering Division
(MSED_) and Center for Theoretical and Computational Materials Science (CTCMS_),
in the Material Measurement Laboratory (MML_) at the National Institute of
Standards and Technology (NIST_).

The solution of coupled sets of PDEs is ubiquitous to the numerical
simulation of science problems.  Numerous PDE solvers exist, using a
variety of languages and numerical approaches. Many are proprietary,
expensive and difficult to customize.  As a result, scientists spend
considerable resources repeatedly developing limited tools for
specific problems.  Our approach, combining the FV method and :term:`Python`,
provides a tool that is extensible, powerful and freely available. A
significant advantage to :term:`Python` is the existing suite of tools for
array calculations, sparse matrices and data rendering.

The :term:`FiPy` framework includes terms for transient diffusion,
convection and standard sources, enabling the solution of arbitrary
combinations of coupled elliptic, hyperbolic and parabolic PDEs. Currently
implemented models include phase field :cite:`BoettingerReview:2002`
:cite:`ChenReview:2002` :cite:`McFaddenReview:2002` treatments of polycrystalline,
dendritic, and electrochemical phase transformations, as well as drug
eluting stents :cite:`Saylor:2011p2794`, reactive wetting :cite:`PhysRevE.82.051601`,
photovoltaics :cite:`Hangarter:2011p2795` and a level set treatment of the
electrodeposition process :cite:`NIST:damascene:2001`.

.. _MML:                  http://www.nist.gov/mml/
.. _CTCMS:                http://www.ctcms.nist.gov/
.. _MSED:                 http://www.nist.gov/mml/msed/
.. _NIST:                 http://www.nist.gov/
"""
from __future__ import unicode_literals
from builtins import input
__docformat__ = 'restructuredtext'

import sys

from fipy.boundaryConditions import *
from fipy.meshes import *
from fipy.solvers import *
from fipy.steppers import *
from fipy.terms import *
from fipy.tools import *
from fipy.variables import *
from fipy.viewers import *

__all__ = []
__all__.extend(boundaryConditions.__all__)
__all__.extend(meshes.__all__)
__all__.extend(solvers.__all__)
__all__.extend(steppers.__all__)
__all__.extend(terms.__all__)
__all__.extend(tools.__all__)
__all__.extend(variables.__all__)
__all__.extend(viewers.__all__)

# fipy needs to export raw_input whether or not parallel

input_original = input

if parallelComm.Nproc > 1:
    def mpi_input(prompt=""):
        parallelComm.Barrier()
        sys.stdout.flush()
        if parallelComm.procID == 0:
            sys.stdout.write(prompt)
            sys.stdout.flush()
            return sys.stdin.readline()
        else:
            return ""
    input = mpi_input

__all__.extend(['input', 'input_original'])

from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

_saved_stdout = sys.stdout

def _serial_doctest_raw_input(prompt):
    """Replacement for `raw_input()` that works in doctests
    """
    _saved_stdout.write("\n")
    _saved_stdout.write(prompt)
    _saved_stdout.flush()
    return sys.stdin.readline()

def doctest_raw_input(prompt):
    """Replacement for `raw_input()` that works in doctests

    This routine attempts to be savvy about running in parallel.
    """
    try:
        from fipy.tools import parallelComm
        parallelComm.Barrier()
        _saved_stdout.flush()
        if parallelComm.procID == 0:
            txt = _serial_doctest_raw_input(prompt)
        else:
            txt = ""
        parallelComm.Barrier()
    except ImportError:
        txt = _serial_doctest_raw_input(prompt)
#     return txt

def test(*args):
    r"""
    Test `Fipy`. Equivalent to::

        $ python setup.py test --modules

    Use

    >>> import fipy
    >>> fipy.test('--help')

    for a full list of options. Options can be passed in the same way
    as they are appended at the command line. For example, to test
    `FiPy` with `Trilinos` and inlining switched on, use

    >>> fipy.test('--trilinos', '--inline')

    At the command line this would be::

        $ python setup.py test --modules --trilinos --inline

    """

    from setuptools import setup
    from fipy.tests.test import test
    import tempfile

    tmpDir = tempfile.mkdtemp()

    try:
        setup(name='FiPy',
              script_args = ['egg_info', '--egg-base=' + tmpDir,
                             'test', '--modules'] + list(args),
              cmdclass={'test': test})
    except SystemExit as exitErr:
        import shutil
        shutil.rmtree(tmpDir)
        raise exitErr

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



"""
Interactively test the viewers
"""
from __future__ import unicode_literals

from fipy.tests.doctestPlus import _LateImportDocTestSuite
import fipy.tests.testProgram

__all__ = []

def _suite():
    return _LateImportDocTestSuite(testModuleNames = (
        'matplotlibViewer.test',
        'mayaviViewer.test',
        ), base = __name__)

if __name__ == '__main__':
    fipy.tests.testProgram.main(defaultTest='_suite')



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.variables.meshVariable import _MeshVariable
from fipy.tools import numerix
from fipy.tools.decorators import deprecate

__all__ = ["CellVariable"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class CellVariable(_MeshVariable):
    """
    Represents the field of values of a variable on a `Mesh`.

    A `CellVariable` can be ``pickled`` to persistent storage (disk) for later use:

    >>> from fipy.meshes import Grid2D
    >>> mesh = Grid2D(dx = 1., dy = 1., nx = 10, ny = 10)

    >>> var = CellVariable(mesh = mesh, value = 1., hasOld = 1, name = 'test')
    >>> x, y = mesh.cellCenters
    >>> var.value = (x * y)

    >>> from fipy.tools import dump
    >>> (f, filename) = dump.write(var, extension = '.gz')
    >>> unPickledVar = dump.read(filename, f)

    >>> print(var.allclose(unPickledVar, atol = 1e-10, rtol = 1e-10))
    1

    """

    def __init__(self, mesh, name='', value=0., rank=None, elementshape=None, unit=None, hasOld=0):
        _MeshVariable.__init__(self, mesh=mesh, name=name, value=value,
                               rank=rank, elementshape=elementshape, unit=unit)

        if hasOld:
            self._old = self.copy()
        else:
            self._old = None

    @property
    def _variableClass(self):
        return CellVariable

    def _OperatorVariableClass(self, baseClass=None):
        """
            >>> from fipy.meshes import Grid1D

            >>> a = CellVariable(mesh=Grid1D(nx=1), value=1)

            >>> c = -a
            >>> b = c.old + 3
            >>> print(b)
            [2]
            >>> from builtins import str
            >>> print(str(b.getsctype()) == str(numerix.NUMERIX.obj2sctype(numerix.array(1))))
            True

        replacing with the same thing is no problem

            >>> a.value = (3)
            >>> b = c.old + 3
            >>> print(b)
            [0]

        replacing with multiple copies causes the reference counting problem

            >>> a.value = (3)
            >>> b = (c + c).old + 3
            >>> print(b)
            [-3]

        the order matters

            >>> b = (c + c).old + 3
            >>> a.value = (2)
            >>> print(b)
            [-1]
        """
        baseClass = _MeshVariable._OperatorVariableClass(self,
                                                         baseClass=baseClass)

        class _CellOperatorVariable(baseClass):
            @property
            def old(self):
                if self._old is None:
                    oldVar = []
                    for v in self.var:
                        if hasattr(v, "old"):
                            oldVar.append(v.old)
                        else:
                            oldVar.append(v)

                    self._old = self.__class__(op=self.op, var=oldVar,
                                               opShape=self.opShape,
                                               canInline=self.canInline)

                return self._old

        return _CellOperatorVariable

    def copy(self):

        return self._getArithmeticBaseClass()(mesh=self.mesh,
                                              name=self.name + "_old",
                                              value=self.value,
                                              hasOld=False)

    @property
    def _globalNumberOfElements(self):
        return self.mesh.globalNumberOfCells

    @property
    def _globalOverlappingIDs(self):
        return self.mesh._globalOverlappingCellIDs

    @property
    def _localNonOverlappingIDs(self):
        return self.mesh._localNonOverlappingCellIDs

    @property
    def globalValue(self):
        """Concatenate and return values from all processors

        When running on a single processor, the result is identical to
        :attr:`~fipy.variables.variable.Variable.value`.
        """
        return self._getGlobalValue(self.mesh._localNonOverlappingCellIDs,
                                    self.mesh._globalNonOverlappingCellIDs)

    def setValue(self, value, unit = None, where = None):
        _MeshVariable.setValue(self, value=self._globalToLocalValue(value), unit=unit, where=where)

    def __call__(self, points=None, order=0, nearestCellIDs=None):
        r"""
        Interpolates the `CellVariable` to a set of points using a
        method that has a memory requirement on the order of `Ncells` by
        `Npoints` in general, but uses only `Ncells` when the
        `CellVariable`'s mesh is a `UniformGrid` object.

        Tests

            >>> from fipy import *
            >>> m = Grid2D(nx=3, ny=2)
            >>> v = CellVariable(mesh=m, value=m.cellCenters[0])
            >>> print(v(((0., 1.1, 1.2), (0., 1., 1.))))
            [ 0.5  1.5  1.5]
            >>> print(v(((0., 1.1, 1.2), (0., 1., 1.)), order=1))
            [ 0.25  1.1   1.2 ]
            >>> m0 = Grid2D(nx=2, ny=2, dx=1., dy=1.)
            >>> m1 = Grid2D(nx=4, ny=4, dx=.5, dy=.5)
            >>> x, y = m0.cellCenters
            >>> v0 = CellVariable(mesh=m0, value=x * y)
            >>> print(v0(m1.cellCenters.globalValue))
            [ 0.25  0.25  0.75  0.75  0.25  0.25  0.75  0.75  0.75  0.75  2.25  2.25
              0.75  0.75  2.25  2.25]
            >>> print(v0(m1.cellCenters.globalValue, order=1))
            [ 0.125  0.25   0.5    0.625  0.25   0.375  0.875  1.     0.5    0.875
              1.875  2.25   0.625  1.     2.25   2.625]

        Parameters
        ----------
        points : tuple or :obj:`list` of :obj:`tuple`
            A point or set of points in the format (X, Y, Z)
        order : {`0`, `1`}
            The order of interpolation, default is 0
        nearestCellIDs : array_like
            Optional argument if user can calculate own
            nearest cell IDs array, shape should be same as points
        """
        if points is not None:

            if nearestCellIDs is None:
                nearestCellIDs = self.mesh._getNearestCellID(points)

            if order == 0:
                return self.globalValue[..., nearestCellIDs]

            elif order == 1:
                ##cellID = self.mesh._getNearestCellID(points)
##                return self[...,self.mesh._getNearestCellID(points)] + numerix.dot(points - self.mesh.cellCenters[...,cellID], self.grad[...,cellID])
                return (self.globalValue[..., nearestCellIDs]
                        + numerix.dot(points - self.mesh.cellCenters.globalValue[..., nearestCellIDs],
                                      self.grad.globalValue[..., nearestCellIDs]))

            else:
                raise ValueError('order should be either 0 or 1')

        else:
            return _MeshVariable.__call__(self)

    @property
    def cellVolumeAverage(self):
        r"""
        Return the cell-volume-weighted average of the `CellVariable`:

        .. math::

           <\phi>_\text{vol}
           = \frac{\sum_\text{cells} \phi_\text{cell} V_\text{cell}}
               {\sum_\text{cells} V_\text{cell}}

        >>> from fipy.meshes import Grid2D
        >>> from fipy.variables.cellVariable import CellVariable
        >>> mesh = Grid2D(nx = 3, ny = 1, dx = .5, dy = .1)
        >>> var = CellVariable(value = (1, 2, 6), mesh = mesh)
        >>> print(var.cellVolumeAverage)
        3.0
        """

        if not hasattr(self, 'volumeAverage'):
            volumes = CellVariable(mesh=self.mesh, value=self.mesh.cellVolumes)
            self.volumeAverage = (self * volumes).sum() / volumes.sum()

        return self.volumeAverage

    @property
    def grad(self):
        r"""
        Return :math:`\nabla \phi` as a rank-1 `CellVariable` (first-order
        gradient).
        """
        return self.gaussGrad

    @property
    def gaussGrad(self):
        r"""
        Return :math:`\frac{1}{V_P} \sum_f \vec{n} \phi_f A_f`
        as a rank-1 `CellVariable` (first-order gradient).

        """
        if not hasattr(self, '_gaussGrad'):
            from fipy.variables.gaussCellGradVariable import _GaussCellGradVariable
            self._gaussGrad = _GaussCellGradVariable(var = self, name = "%s_gauss_grad" % self.name)

        return self._gaussGrad

    @property
    def leastSquaresGrad(self):
        r"""
        Return :math:`\nabla \phi`, which is determined by solving for :math:`\nabla \phi`
        in the following matrix equation,

        .. math::

           \nabla \phi \cdot \sum_f d_{AP}^2 \vec{n}_{AP} \otimes \vec{n}_{AP} =
           \sum_f d_{AP}^2 \left( \vec{n} \cdot \nabla \phi \right)_{AP}

        The matrix equation is derived by minimizing
        the following least squares sum,

        .. math::

           F \left( \phi_x, \phi_y \right) = \sqrt{\sum_f \left( d_{AP}
           \vec{n}_{AP} \cdot \nabla \phi - d_{AP} \left( \vec{n}_{AP} \cdot
           \nabla \phi \right)_{AP} \right)^2 }

        Tests

        >>> from fipy import Grid2D
        >>> m = Grid2D(nx=2, ny=2, dx=0.1, dy=2.0)
        >>> print(numerix.allclose(CellVariable(mesh=m, value=(0, 1, 3, 6)).leastSquaresGrad.globalValue, \
        ...                                     [[8.0, 8.0, 24.0, 24.0],
        ...                                      [1.2, 2.0, 1.2, 2.0]]))
        True

        >>> from fipy import Grid1D
        >>> print(numerix.allclose(CellVariable(mesh=Grid1D(dx=(2.0, 1.0, 0.5)),
        ...                                     value=(0, 1, 2)).leastSquaresGrad.globalValue, [[0.461538461538, 0.8, 1.2]]))
        True
        """

        if not hasattr(self, '_leastSquaresGrad'):
            from fipy.variables.leastSquaresCellGradVariable import _LeastSquaresCellGradVariable
            self._leastSquaresGrad = _LeastSquaresCellGradVariable(var = self,
                    name = "%s_least_squares_grad" % self.name)

        return self._leastSquaresGrad

    @property
    def arithmeticFaceValue(self):
        r"""
        Returns a `FaceVariable` whose value corresponds to the arithmetic interpolation
        of the adjacent cells:

        .. math::

           \phi_f = (\phi_1 - \phi_2) \frac{d_{f2}}{d_{12}} + \phi_2

        >>> from fipy.meshes import Grid1D
        >>> from fipy import numerix
        >>> mesh = Grid1D(dx = (1., 1.))
        >>> L = 1
        >>> R = 2
        >>> var = CellVariable(mesh = mesh, value = (L, R))
        >>> faceValue = var.arithmeticFaceValue[mesh.interiorFaces.value]
        >>> answer = (R - L) * (0.5 / 1.) + L
        >>> print(numerix.allclose(faceValue, answer, atol = 1e-10, rtol = 1e-10))
        True

        >>> mesh = Grid1D(dx = (2., 4.))
        >>> var = CellVariable(mesh = mesh, value = (L, R))
        >>> faceValue = var.arithmeticFaceValue[mesh.interiorFaces.value]
        >>> answer = (R - L) * (1.0 / 3.0) + L
        >>> print(numerix.allclose(faceValue, answer, atol = 1e-10, rtol = 1e-10))
        True

        >>> mesh = Grid1D(dx = (10., 100.))
        >>> var = CellVariable(mesh = mesh, value = (L, R))
        >>> faceValue = var.arithmeticFaceValue[mesh.interiorFaces.value]
        >>> answer = (R - L) * (5.0 / 55.0) + L
        >>> print(numerix.allclose(faceValue, answer, atol = 1e-10, rtol = 1e-10))
        True
        """
        if not hasattr(self, '_arithmeticFaceValue'):
            from fipy.variables.arithmeticCellToFaceVariable import _ArithmeticCellToFaceVariable
            self._arithmeticFaceValue = _ArithmeticCellToFaceVariable(self)

        return self._arithmeticFaceValue

    faceValue = arithmeticFaceValue

    @property
    def minmodFaceValue(self):
        r"""
        Returns a `FaceVariable` with a value that is the minimum of
        the absolute values of the adjacent cells. If the values are
        of opposite sign then the result is zero:

        .. math::

           \phi_f = \begin{cases}
                          \phi_1& \text{when $|\phi_1| \le |\phi_2|$},\\
                          \phi_2& \text{when $|\phi_2| < |\phi_1|$},\\
                          0 & \text{when $\phi1 \phi2 < 0$}
                    \end{cases}

        >>> from fipy import *
        >>> print(CellVariable(mesh=Grid1D(nx=2), value=(1, 2)).minmodFaceValue)
        [1 1 2]
        >>> print(CellVariable(mesh=Grid1D(nx=2), value=(-1, -2)).minmodFaceValue)
        [-1 -1 -2]
        >>> print(CellVariable(mesh=Grid1D(nx=2), value=(-1, 2)).minmodFaceValue)
        [-1  0  2]
        """
        if not hasattr(self, '_minmodFaceValue'):
            from fipy.variables.minmodCellToFaceVariable import _MinmodCellToFaceVariable
            self._minmodFaceValue = _MinmodCellToFaceVariable(self)

        return self._minmodFaceValue

    @property
    def harmonicFaceValue(self):
        r"""
        Returns a `FaceVariable` whose value corresponds to the harmonic interpolation
        of the adjacent cells:

        .. math::

           \phi_f = \frac{\phi_1 \phi_2}{(\phi_2 - \phi_1) \frac{d_{f2}}{d_{12}} + \phi_1}

        >>> from fipy.meshes import Grid1D
        >>> from fipy import numerix
        >>> mesh = Grid1D(dx = (1., 1.))
        >>> L = 1
        >>> R = 2
        >>> var = CellVariable(mesh = mesh, value = (L, R))
        >>> faceValue = var.harmonicFaceValue[mesh.interiorFaces.value]
        >>> answer = L * R / ((R - L) * (0.5 / 1.) + L)
        >>> print(numerix.allclose(faceValue, answer, atol = 1e-10, rtol = 1e-10))
        True

        >>> mesh = Grid1D(dx = (2., 4.))
        >>> var = CellVariable(mesh = mesh, value = (L, R))
        >>> faceValue = var.harmonicFaceValue[mesh.interiorFaces.value]
        >>> answer = L * R / ((R - L) * (1.0 / 3.0) + L)
        >>> print(numerix.allclose(faceValue, answer, atol = 1e-10, rtol = 1e-10))
        True

        >>> mesh = Grid1D(dx = (10., 100.))
        >>> var = CellVariable(mesh = mesh, value = (L, R))
        >>> faceValue = var.harmonicFaceValue[mesh.interiorFaces.value]
        >>> answer = L * R / ((R - L) * (5.0 / 55.0) + L)
        >>> print(numerix.allclose(faceValue, answer, atol = 1e-10, rtol = 1e-10))
        True
        """
        if not hasattr(self, '_harmonicFaceValue'):
            from fipy.variables.harmonicCellToFaceVariable import _HarmonicCellToFaceVariable
            self._harmonicFaceValue = _HarmonicCellToFaceVariable(self)

        return self._harmonicFaceValue

    @property
    def faceGrad(self):
        r"""
        Return :math:`\nabla \phi` as a rank-1 `FaceVariable` using differencing
        for the normal direction(second-order gradient).
        """
        if not hasattr(self, '_faceGrad'):
            from fipy.variables.faceGradVariable import _FaceGradVariable
            self._faceGrad = _FaceGradVariable(self)

        return self._faceGrad

    @property
    @deprecate(new_name="grad.arithmeticFaceValue", version=3.3)
    def faceGradAverage(self):
        r"""
        Return :math:`\nabla \phi` as a rank-1 `FaceVariable` using averaging
        for the normal direction(second-order gradient)
        """
        return self.grad.arithmeticFaceValue

    @property
    def old(self):
        """
        Return the values of the `CellVariable` from the previous
        solution sweep.

        Combinations of `CellVariable's` should also return old
        values.

        >>> from fipy.meshes import Grid1D
        >>> mesh = Grid1D(nx = 2)
        >>> from fipy.variables.cellVariable import CellVariable
        >>> var1 = CellVariable(mesh = mesh, value = (2, 3), hasOld = 1)
        >>> var2 = CellVariable(mesh = mesh, value = (3, 4))
        >>> v = var1 * var2
        >>> print(v)
        [ 6 12]
        >>> var1.value = ((3, 2))
        >>> print(v)
        [9 8]
        >>> print(v.old)
        [ 6 12]

        The following small test is to correct for a bug when the
        operator does not just use variables.

        >>> v1 = var1 * 3
        >>> print(v1)
        [9 6]
        >>> print(v1.old)
        [6 9]
        """
        if self._old is None:
            return self
        else:
            return self._old
##             import weakref
##          return weakref.proxy(self._old)

    def updateOld(self):
        """
        Set the values of the previous solution sweep to the current
        values.

        >>> from fipy import *
        >>> v = CellVariable(mesh=Grid1D(), hasOld=False)
        >>> v.updateOld()
        Traceback (most recent call last):
           ...
        AssertionError: The updateOld method requires the CellVariable to have an old value. Set hasOld to True when instantiating the CellVariable.

        """
        if self._old is None:
            raise AssertionError('The updateOld method requires the CellVariable to have an old value. Set hasOld to True when instantiating the CellVariable.')
        else:
            self._old.value = self.value.copy()

    def _resetToOld(self):
        if self._old is not None:
            self.value = (self._old.value)

    def _getShapeFromMesh(mesh):
        """
        Return the shape of this variable type, given a particular mesh.
        """
        return (mesh.numberOfCells,)

    _getShapeFromMesh = staticmethod(_getShapeFromMesh)

    def _getArithmeticBaseClass(self, other = None):
        """
        Given `self` and `other`, return the desired base
        class for an operation result.
        """
        if other is None:
            return CellVariable

        return _MeshVariable._getArithmeticBaseClass(self, other)

##pickling

    def __getstate__(self):
        """
        Used internally to collect the necessary information to ``pickle`` the
        `CellVariable` to persistent storage.
        """
        return {
            'mesh' : self.mesh,
            'name' : self.name,
            'value' : self.globalValue,
            'unit' : self.unit,
            'old' : self._old
        }

    def __setstate__(self, dict):
        """
        Used internally to create a new `CellVariable` from ``pickled``
        persistent storage.
        """

        import sys
        self._refcount = sys.getrefcount(self)

        hasOld = 0
        if dict['old'] is not None:
            hasOld = 1

        self.__init__(mesh=dict['mesh'], name=dict['name'], value=dict['value'], unit=dict['unit'], hasOld=hasOld)
##         self.__init__(hasOld=hasOld, **dict)
        if self._old is not None:
            self._old.value = (dict['old'].value)

    def constrain(self, value, where=None):
        r"""
        Constrains the `CellVariable` to `value` at a location specified by `where`.

            >>> from fipy import *
            >>> m = Grid1D(nx=3)
            >>> v = CellVariable(mesh=m, value=m.cellCenters[0])
            >>> v.constrain(0., where=m.facesLeft)
            >>> v.faceGrad.constrain([1.], where=m.facesRight)
            >>> print(v.faceGrad)
            [[ 1.  1.  1.  1.]]
            >>> print(v.faceValue)
            [ 0.   1.   2.   2.5]

        Changing the constraint changes the dependencies

            >>> v.constrain(1., where=m.facesLeft)
            >>> print(v.faceGrad)
            [[-1.  1.  1.  1.]]
            >>> print(v.faceValue)
            [ 1.   1.   2.   2.5]

        Constraints can be `Variable`

            >>> c = Variable(0.)
            >>> v.constrain(c, where=m.facesLeft)
            >>> print(v.faceGrad)
            [[ 1.  1.  1.  1.]]
            >>> print(v.faceValue)
            [ 0.   1.   2.   2.5]
            >>> c.value = 1.
            >>> print(v.faceGrad)
            [[-1.  1.  1.  1.]]
            >>> print(v.faceValue)
            [ 1.   1.   2.   2.5]

        Constraints can have a `Variable` mask.

            >>> v = CellVariable(mesh=m)
            >>> mask = FaceVariable(mesh=m, value=m.facesLeft)
            >>> v.constrain(1., where=mask)
            >>> print(v.faceValue)
            [ 1.  0.  0.  0.]
            >>> mask[:] = mask | m.facesRight
            >>> print(v.faceValue)
            [ 1.  0.  0.  1.]

        """
        from fipy.boundaryConditions.constraint import Constraint
        if not isinstance(value, Constraint):
            value = Constraint(value=value, where=where)

        if numerix.shape(value.where)[-1] == self.mesh.numberOfFaces:

            if not hasattr(self, 'faceConstraints'):
                self.faceConstraints = []
            self.faceConstraints.append(value)
            self._requires(value.value)
            # self._requires(value.where) ???
            self._markStale()
        else:
##            _MeshVariable.constrain(value, where)
            super(CellVariable, self).constrain(value, where)

    def release(self, constraint):
        """Remove `constraint` from `self`

        >>> from fipy import *
        >>> m = Grid1D(nx=3)
        >>> v = CellVariable(mesh=m, value=m.cellCenters[0])
        >>> c = Constraint(0., where=m.facesLeft)
        >>> v.constrain(c)
        >>> print(v.faceValue)
        [ 0.   1.   2.   2.5]
        >>> v.release(constraint=c)
        >>> print(v.faceValue)
        [ 0.5  1.   2.   2.5]
        """
        try:
            _MeshVariable.release(self, constraint=constraint)
        except ValueError:
            self.faceConstraints.remove(constraint)

    def _test(self):
        """
        Tests

        >>> from fipy import *
        >>> m = Grid1D(nx=6)
        >>> q = CellVariable(mesh=m, elementshape=(2,))
        >>> print(numerix.allequal(q.faceGrad.globalValue.shape, (1, 2, 7)))
        True
        >>> from fipy import *
        >>> m = Grid2D(nx=3, ny=3)
        >>> x, y = m.cellCenters
        >>> v = CellVariable(mesh=m, elementshape=(3,))
        >>> v[0] = x
        >>> v[1] = y
        >>> v[2] = x**2
        >>> print(v.faceGrad.allclose([[[ 0.5, 1.,  0.5, 0.5, 1.,  0.5, 0.5, 1.,  0.5, 0.5, 1.,  0.5, 0.,  1.,  1.,
        ...                               0.,  0.,  1.,  1.,  0.,  0.,  1.,  1.,  0. ],
        ...                             [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
        ...                               0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0. ],
        ...                             [ 1.,  3.,  2.,  1.,  3.,  2.,  1.,  3.,  2.,  1.,  3.,  2.,  0.,  2.,  4.,
        ...                               0.,  0.,  2.,  4.,  0.,  0.,  2.,  4.,  0. ]],
        ...                            [[ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
        ...                               0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0. ],
        ...                             [ 0.,  0.,  0.,  1.,  1.,  1.,  1.,  1.,  1.,  0.,  0.,  0.,  0.5, 0.5,
        ...                               0.5, 0.5, 1.,  1.,  1.,  1.,  0.5, 0.5, 0.5, 0.5],
        ...                             [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
        ...                               0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0. ]]]))
        True
        >>> print(v.grad)
        [[[ 0.5  1.   0.5  0.5  1.   0.5  0.5  1.   0.5]
          [ 0.   0.   0.   0.   0.   0.   0.   0.   0. ]
          [ 1.   3.   2.   1.   3.   2.   1.   3.   2. ]]
        <BLANKLINE>
         [[ 0.   0.   0.   0.   0.   0.   0.   0.   0. ]
          [ 0.5  0.5  0.5  1.   1.   1.   0.5  0.5  0.5]
          [ 0.   0.   0.   0.   0.   0.   0.   0.   0. ]]]

        """

class _ReMeshedCellVariable(CellVariable):
    def __init__(self, oldVar, newMesh):
        newValues = oldVar.getValue(points = newMesh.cellCenters)
        CellVariable.__init__(self, newMesh, name = oldVar.name, value = newValues, unit = oldVar.unit)

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()



from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from builtins import range
__docformat__ = 'restructuredtext'

from fipy.solvers.pysparse.pysparseSolver import PysparseSolver
from fipy.matrices.pysparseMatrix import _PysparseMatrixFromShape

__all__ = ["LinearJORSolver"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class LinearJORSolver(PysparseSolver):
    """

    The `LinearJORSolver` solves a linear system of equations using
    Jacobi over-relaxation. This method solves systems with a general
    non-symmetric coefficient matrix.

    """
    def __init__(self, tolerance=1e-10, iterations=1000, relaxation=1.0):
        """
        The `Solver` class should not be invoked directly.

        Parameters
        ----------
        tolerance : float
            Required error tolerance.
        iterations : int
            Maximum number of iterative steps to perform.
        relaxation : float
            Fraction of update to apply
        """
        super(LinearJORSolver, self).__init__(tolerance=tolerance,
                                              iterations=iterations)
        self.relaxation = relaxation

    def _solve_(self, L, x, b):

        d = L.takeDiagonal()
        D = _PysparseMatrixFromShape(size=len(d))
        D.putDiagonal(d)

        LU = L - D
        tol = 1e+10
        xold = x.copy()

        for iteration in range(self.iterations):
            if tol <= self.tolerance:
                break

            residual = L * x - b

            xold[:] = x
            x[:] = (-(LU) * x + b) / d

            x[:] = xold + self.relaxation * (x - xold)

            tol = max(abs(residual))

            print(iteration, tol)



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

import os
from fipy.solvers.pysparseMatrixSolver import _PysparseMatrixSolver

__all__ = ["PysparseSolver"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class PysparseSolver(_PysparseMatrixSolver):
    """
    The base `pysparseSolver` class.

    .. attention:: This class is abstract. Always create one of its subclasses.
    """
    def __init__(self, *args, **kwargs):
        if self.__class__ is PysparseSolver:
            raise NotImplementedError("can't instantiate abstract base class")

        super(PysparseSolver, self).__init__(*args, **kwargs)

    def _solve_(self, L, x, b):
        """
        `_solve_` is only for use by solvers which may use
        preconditioning. If you are writing a solver which
        doesn't use preconditioning, this must be overridden.

        Parameters
        ----------
        L : ~fipy.matrices.pysparseMatrix._PysparseMeshMatrix
            Matrix
        x : ndarray
            Solution vector
        b : ndarray
            Right hand side vector
        """

        A = L.matrix

        if self.preconditioner is None:
            P = None
        else:
            P, A = self.preconditioner._applyToMatrix(A)

        info, iter, relres = self.solveFnc(A, b, x, self.tolerance,
                                           self.iterations, P)

        self._raiseWarning(info, iter, relres)

        if 'FIPY_VERBOSE_SOLVER' in os.environ:
            from fipy.tools.debug import PRINT
            PRINT('iterations: %d / %d' % (iter, self.iterations))

            if info < 0:
                PRINT('failure', self._warningList[info].__class__.__name__)
            PRINT('relres:', relres)

    def _solve(self):

        if self.var.mesh.communicator.Nproc > 1:
            raise Exception("Pysparse solvers cannot be used with multiple processors")

        array = self.var.numericValue.ravel()

        from fipy.terms import SolutionVariableNumberError

        if ((self.matrix == 0)
            or (self.matrix.matrix.shape[0] != self.matrix.matrix.shape[1])
            or (self.matrix.matrix.shape[0] != len(array))):

            raise SolutionVariableNumberError

        self._solve_(self.matrix, array, self.RHSvector)
        factor = self.var.unit.factor
        if factor != 1:
            array /= self.var.unit.factor

        self.var[:] = array.reshape(self.var.shape)



from __future__ import division
from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.tools import numerix

from fipy.viewers.matplotlibViewer.matplotlib2DViewer import AbstractMatplotlib2DViewer

__all__ = ["Matplotlib2DGridContourViewer"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class Matplotlib2DGridContourViewer(AbstractMatplotlib2DViewer):
    """Displays a contour plot of a 2D `CellVariable` object.

    The `Matplotlib2DGridContourViewer` plots a 2D `CellVariable` using Matplotlib_.

    .. _Matplotlib: http://matplotlib.sourceforge.net/
    """

    __doc__ += AbstractMatplotlib2DViewer._test2D(viewer="Matplotlib2DGridContourViewer")


    def __init__(self, vars, title=None, limits={}, cmap=None, colorbar='vertical', axes=None, figaspect='auto', **kwlimits):
        """Creates a `Matplotlib2DViewer`.

        Parameters
        ----------
        vars : ~fipy.variables.cellVariable.CellVariable
            the `Variable` to display.
        title : str, optional
            displayed at the top of the `Viewer` window
        limits : dict
          a (deprecated) alternative to limit keyword arguments
        float xmin, xmax, ymin, ymax, datamin, datamax : float, optional
            displayed range of data. Any limit set to
            a (default) value of `None` will autoscale.
        cmap : ~matplotlib.colors.Colormap, optional
            the :class:`~matplotlib.colors.Colormap`.
            Defaults to `matplotlib.cm.jet`
        colorbar : bool, optional
            plot a color bar if not `None`
        axes : ~matplotlib.axes.Axes, optional
            if not `None`, `vars` will be plotted into this Matplotlib `Axes` object
        figaspect : float, optional
            desired aspect ratio of figure. If a number, use that aspect
            ratio. If `auto`, the aspect ratio will be determined from
            the *vars*'s mesh.
        """
        kwlimits.update(limits)
        AbstractMatplotlib2DViewer.__init__(self, vars=vars, title=title,
                                            cmap=cmap, colorbar=colorbar, axes=axes, figaspect=figaspect,
                                            **kwlimits)

        self._plot()

    def _getSuitableVars(self, vars):
        from fipy.meshes.nonUniformGrid2D import NonUniformGrid2D
        from fipy.meshes.uniformGrid2D import UniformGrid2D
        from fipy.variables.cellVariable import CellVariable
        vars = [var for var in AbstractMatplotlib2DViewer._getSuitableVars(self, vars) \
          if ((isinstance(var.mesh, NonUniformGrid2D)
               or isinstance(var.mesh, UniformGrid2D))
              and isinstance(var, CellVariable))]
        if len(vars) == 0:
            from fipy.viewers import MeshDimensionError
            raise MeshDimensionError("The mesh must be a Grid2D instance")
        # this viewer can only display one variable
        return [vars[0]]

    def _plot(self):
        super(Matplotlib2DGridContourViewer, self)._plot()

##         plt.clf()

##         ## Added garbage collection since matplotlib objects seem to hang
##         ## around and accumulate.
##         import gc
##         gc.collect()

        mesh = self.vars[0].mesh
        shape = mesh.shape
        X, Y = mesh.cellCenters
        Z = self.vars[0].value
        X, Y, Z = [v.reshape(shape, order='F') for v in (X, Y, Z)]

        numberOfContours = 10
        smallNumber = 1e-7
        zmin = self.norm.vmin
        zmax = self.norm.vmax
        diff = zmax - zmin

        if diff < smallNumber:
            V = numerix.arange(numberOfContours + 1) * smallNumber / numberOfContours + zmin
        else:
            V = numerix.arange(numberOfContours + 1) * diff / numberOfContours + zmin

        self.axes.contourf(X, Y, Z, V, cmap=self.cmap)

        self.axes.set_xlim(xmin=self._getLimit('xmin'),
                           xmax=self._getLimit('xmax'))

        self.axes.set_ylim(ymin=self._getLimit('ymin'),
                           ymax=self._getLimit('ymax'))

def _test():
    from fipy.viewers.viewer import _test2D
    _test2D(Matplotlib2DGridContourViewer)

if __name__ == "__main__":
    import fipy.tests.doctestPlus
    fipy.tests.doctestPlus.execButNoTest()



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.viewers.matplotlibViewer.matplotlib1DViewer import *
from fipy.viewers.matplotlibViewer.matplotlib2DGridViewer import *
from fipy.viewers.matplotlibViewer.matplotlib2DGridContourViewer import *
from fipy.viewers.matplotlibViewer.matplotlib2DViewer import *
from fipy.viewers.matplotlibViewer.matplotlibVectorViewer import *
from fipy.viewers.matplotlibViewer.matplotlibStreamViewer import *

__all__ = ["MatplotlibViewer"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]
__all__.extend(matplotlib1DViewer.__all__)
__all__.extend(matplotlib2DGridViewer.__all__)
__all__.extend(matplotlib2DGridContourViewer.__all__)
__all__.extend(matplotlib2DViewer.__all__)
__all__.extend(matplotlibVectorViewer.__all__)
__all__.extend(matplotlibStreamViewer.__all__)

def MatplotlibViewer(vars, title=None, limits={}, cmap=None, colorbar='vertical', axes=None, **kwlimits):
    """Generic function for creating a `MatplotlibViewer`.

    The `MatplotlibViewer` factory will search the module tree and return an
    instance of the first `MatplotlibViewer` it finds of the correct dimension
    and rank.

    It is possible to view different
    :class:`~fipy.variables.variable.Variable`\s against different Matplotlib_
    `Axes`

    >>> from matplotlib import pyplot as plt
    >>> from fipy import *

    >>> plt.ion()
    >>> fig = plt.figure()

    >>> ax1 = plt.subplot((221))
    >>> ax2 = plt.subplot((223))
    >>> ax3 = plt.subplot((224))

    >>> k = Variable(name="k", value=0.)

    >>> mesh1 = Grid1D(nx=100)
    >>> x, = mesh1.cellCenters
    >>> xVar = CellVariable(mesh=mesh1, name="x", value=x)
    >>> viewer1 = MatplotlibViewer(vars=(numerix.sin(0.1 * k * xVar), numerix.cos(0.1 * k * xVar / numerix.pi)),
    ...                            limits={'xmin': 10, 'xmax': 90},
    ...                            datamin=-0.9, datamax=2.0,
    ...                            title="Grid1D test",
    ...                            axes=ax1,
    ...                            legend=None)

    >>> mesh2 = Grid2D(nx=50, ny=100, dx=0.1, dy=0.01)
    >>> x, y = mesh2.cellCenters
    >>> xyVar = CellVariable(mesh=mesh2, name="x y", value=x * y)
    >>> viewer2 = MatplotlibViewer(vars=numerix.sin(k * xyVar),
    ...                            limits={'ymin': 0.1, 'ymax': 0.9},
    ...                            datamin=-0.9, datamax=2.0,
    ...                            title="Grid2D test",
    ...                            axes=ax2,
    ...                            colorbar=None)

    >>> mesh3 = (Grid2D(nx=5, ny=10, dx=0.1, dy=0.1)
    ...          + (Tri2D(nx=5, ny=5, dx=0.1, dy=0.1)
    ...             + ((0.5,), (0.2,))))
    >>> x, y = mesh3.cellCenters
    >>> xyVar = CellVariable(mesh=mesh3, name="x y", value=x * y)
    >>> viewer3 = MatplotlibViewer(vars=numerix.sin(k * xyVar),
    ...                            limits={'ymin': 0.1, 'ymax': 0.9},
    ...                            datamin=-0.9, datamax=2.0,
    ...                            title="Irregular 2D test",
    ...                            axes=ax3,
    ...                            cmap = plt.cm.OrRd)

    >>> viewer = MultiViewer(viewers=(viewer1, viewer2, viewer3))
    >>> from builtins import range
    >>> for kval in range(10):
    ...     k.setValue(kval)
    ...     viewer.plot()

    >>> viewer._promptForOpinion()

    .. _Matplotlib: http://matplotlib.sourceforge.net/

    Parameters
    ----------
    vars : ~fipy.variables.cellVariable.CellVariable or list
        the `Variable` objects to display.
    title : str, optional
        displayed at the top of the `Viewer` window
    limits : dict
        a (deprecated) alternative to limit keyword arguments
    float xmin, xmax, ymin, ymax, datamin, datamax : float, optional
        displayed range of data. A 1D `Viewer` will only use *xmin* and
        *xmax*, a 2D viewer will also use *ymin* and *ymax*. All
        viewers will use *datamin* and *datamax*. Any limit set to a
        (default) value of `None` will autoscale.
    cmap : ~matplotlib.colors.Colormap, optional
        the :class:`~matplotlib.colors.Colormap`.
        Defaults to `matplotlib.cm.jet`
    colorbar : bool, optional
        plot a color bar in specified orientation if not `None`
    axes : ~matplotlib.axes.Axes, optional
        if not `None`, `vars` will be plotted into this Matplotlib `Axes` object

    """
    if type(vars) not in [type([]), type(())]:
        vars = [vars]

    kwlimits.update(limits)

    from fipy.viewers import MeshDimensionError
    from fipy.viewers.matplotlibViewer import (Matplotlib1DViewer,
                                               Matplotlib2DGridViewer,
                                               Matplotlib2DViewer,
                                               MatplotlibVectorViewer)

    try:
        return Matplotlib1DViewer(vars=vars, title=title, axes=axes, **kwlimits)
    except MeshDimensionError:
        try:
            from fipy.viewers.matplotlibViewer.matplotlib2DGridViewer import Matplotlib2DGridViewer
            return Matplotlib2DGridViewer(vars=vars, title=title, cmap=cmap, colorbar=colorbar, axes=axes, **kwlimits)
        except MeshDimensionError:
            try:
                from fipy.viewers.matplotlibViewer.matplotlib2DViewer import Matplotlib2DViewer
                return Matplotlib2DViewer(vars=vars, title=title, cmap=cmap, colorbar=colorbar, axes=axes, **kwlimits)
            except MeshDimensionError:
                from fipy.viewers.matplotlibViewer.matplotlibVectorViewer import MatplotlibVectorViewer
                return MatplotlibVectorViewer(vars=vars, title=title, axes=axes, **kwlimits)



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

from fipy.terms.sourceTerm import SourceTerm

class _ExplicitSourceTerm(SourceTerm):
    r"""

    The `_ExplicitSourceTerm` discretization is given by

    .. math::

       \int_V S \,dV \simeq S_P V_P

    where :math:`S` is the `coeff` value. This source is added to the RHS vector and
    does not contribute to the solution matrix.

    """

    def _getWeight(self, var, transientGeomCoeff=None, diffusionGeomCoeff=None):
        return {
            'b vector': -1,
            'new value': 0,
            'old value': 0,
            'diagonal' : 0
        }

    def __repr__(self):
        return repr(self.coeff)



from __future__ import unicode_literals
from builtins import range
__docformat__ = 'restructuredtext'

import sys

from fipy.tools import numerix
from fipy.viewers.viewer import AbstractViewer
from fipy.variables.cellVariable import CellVariable
from fipy.variables.faceVariable import FaceVariable

__all__ = ["TSVViewer"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class TSVViewer(AbstractViewer):
    """
    "Views" one or more variables in tab-separated-value format.

    Output is a list of coordinates and variable values at each cell center.

    File contents will be, e.g.::

        title
        x	y	...	var0	var2	...
        0.0	0.0	...	3.14	1.41	...
        1.0	0.0	...	2.72	0.866	...
        :
        :

    """
    _axis = ["x", "y", "z"]

    def __init__(self, vars, title=None, limits={}, **kwlimits):
        """
        Creates a `TSVViewer`.

        Any cell centers that lie outside the limits provided will not be included.
        Any values that lie outside the `datamin` or `datamax` will be
        replaced with `nan`.

        All variables must have the same mesh.

        It tries to do something reasonable with rank-1 `CellVariable` and `FaceVariable` objects.

        Parameters
        ----------
        vars : ~fipy.variables.cellVariable.CellVariable or ~fipy.variables.faceVariable.FaceVariable or list
            the `MeshVariable` objects to display.
        title : str, optional
            displayed at the top of the `Viewer` window
        limits : dict, optional
            a (deprecated) alternative to limit keyword arguments
        float xmin, xmax, ymin, ymax, zmin, zmax, datamin, datamax : float, optional
            displayed range of data. Any limit set to
            a (default) value of `None` will autoscale.
        """
        kwlimits.update(limits)
        AbstractViewer.__init__(self, vars=vars, title=title, **kwlimits)

        mesh = self.vars[0].mesh

        for var in self.vars:
            assert mesh is var.mesh


    def _plot(self, values, f, dim):
        for index in range(values.shape[-1]):
            lineValues = values[..., index]

            # omit any elements whose cell centers lie outside of the specified limits
            skip = False
            for axis in range(dim):
                mini = self._getLimit("%smin" % self._axis[axis])
                maxi = self._getLimit("%smax" % self._axis[axis])

                if (mini and lineValues[axis] < mini) or (maxi and lineValues[axis] > maxi):
                    skip = True
                    break

            if skip:
                continue

            # replace any values that lie outside of the specified datalimits with 'nan'
            for valIndex in range(dim, len(lineValues)):
                mini = self._getLimit("datamin")
                maxi = self._getLimit("datamax")

                if (mini and lineValues[valIndex] < mini) or (maxi and lineValues[valIndex] > maxi):
                    lineValues[valIndex] = float("NaN")
                    break

            line = ["%.15g" % value for value in lineValues]
            f.write("\t".join(line))
            f.write("\n")

    def plot(self, filename=None):
        """
        "plot" the coordinates and values of the variables to `filename`.
        If `filename` is not provided, "plots" to `stdout`.

        >>> from fipy.meshes import Grid1D
        >>> m = Grid1D(nx = 3, dx = 0.4)
        >>> from fipy.variables.cellVariable import CellVariable
        >>> v = CellVariable(mesh = m, name = "var", value = (0, 2, 5))
        >>> TSVViewer(vars = (v, v.grad)).plot() #doctest: +NORMALIZE_WHITESPACE
        x       var     var_gauss_grad_x
        0.2     0       2.5
        0.6     2       6.25
        1       5       3.75

        >>> from fipy.meshes import Grid2D
        >>> m = Grid2D(nx = 2, dx = .1, ny = 2, dy = 0.3)
        >>> v = CellVariable(mesh = m, name = "var", value = (0, 2, -2, 5))
        >>> TSVViewer(vars = (v, v.grad)).plot() #doctest: +NORMALIZE_WHITESPACE
        x       y       var     var_gauss_grad_x        var_gauss_grad_y
        0.05    0.15    0       10      -3.33333333333333
        0.15    0.15    2       10      5
        0.05    0.45    -2      35      -3.33333333333333
        0.15    0.45    5       35      5

        Parameters
        ----------
        filename : str
            If not `None`, the name of a file to save the image into.
        """

        mesh = self.vars[0].mesh
        dim = mesh.dim

        if filename is not None:
            import os
            if mesh.communicator.procID == 0:
                if os.path.splitext(filename)[1] == ".gz":
                    import gzip
                    f = gzip.GzipFile(filename = filename, mode = 'w', fileobj = None)
                else:
                    f = open(filename, "w")
            else:
                f = open(os.devnull, mode='w')
        else:
            f = sys.stdout

        if self.title and len(self.title) > 0:
            f.write(self.title)
            f.write("\n")

        headings = []
        for index in range(dim):
            headings.extend(self._axis[index])

        for var in self.vars:
            name = var.name
            if (isinstance(var, CellVariable) or isinstance(var, FaceVariable)) and var.rank == 1:
                for index in range(dim):
                    headings.extend(["%s_%s" % (name, self._axis[index])])
            else:
                headings.extend([name])

        f.write("\t".join(headings))
        f.write("\n")

        cellVars = [var for var in self.vars if isinstance(var, CellVariable)]
        faceVars = [var for var in self.vars if isinstance(var, FaceVariable)]

        if len(cellVars) > 0:
            values = mesh.cellCenters.globalValue
            for var in self.vars:
                if isinstance(var, CellVariable) and var.rank == 1:
                    values = numerix.concatenate((values, numerix.array(var.globalValue)))
                else:
                    values = numerix.concatenate((values, (numerix.array(var.globalValue),)))

            self._plot(values, f, dim)

        if len(faceVars) > 0:
            values = mesh.faceCenters.globalValue
            for var in self.vars:
                if isinstance(var, FaceVariable) and var.rank == 1:
                    values = numerix.concatenate((values, numerix.array(var.globalValue)))
                else:
                    values = numerix.concatenate((values, (numerix.array(var.globalValue),)))

            self._plot(values, f, dim)

        if f is not sys.stdout:
            f.close()

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

from fipy.terms.abstractBinaryTerm import _AbstractBinaryTerm
from fipy.variables.coupledCellVariable import _CoupledCellVariable
from fipy.variables.cellVariable import CellVariable
from fipy.tools import numerix
from fipy.terms import SolutionVariableNumberError
from fipy.matrices.offsetSparseMatrix import OffsetSparseMatrix

class _CoupledBinaryTerm(_AbstractBinaryTerm):
    """
    Test to ensure that `_getTransientGeomCoeff` and `_getDiffusionGeomCoeff` return sensible results for coupled equations.

    >>> from fipy import *
    >>> m = Grid1D(nx=1)
    >>> v0 = CellVariable(mesh=m)
    >>> v1 = CellVariable(mesh=m)
    >>> eq0 = TransientTerm(1, var=v0) == DiffusionTerm(2, var=v1)
    >>> eq1 = TransientTerm(3, var=v1) == DiffusionTerm(4, var=v0)
    >>> eq = eq0 & eq1
    >>> print(eq._getTransientGeomCoeff(v0))
    None
    >>> print(eq._getDiffusionGeomCoeff(v1))
    None
    >>> tranCoeff = eq._uncoupledTerms[0]._getTransientGeomCoeff(v0)
    >>> print(numerix.allequal(tranCoeff, [1])) # doctest: +PROCESSOR_0
    True
    >>> diffCoeff = eq._uncoupledTerms[1]._getDiffusionGeomCoeff(v0)
    >>> print(numerix.allequal(diffCoeff, [[-8, -8]]))
    True

    """
    def __init__(self, term, other):
        _AbstractBinaryTerm.__init__(self, term, other)
        if len(self._vars) < len(self._uncoupledTerms):
            raise SolutionVariableNumberError

    @property
    def _uncoupledTerms(self):
        return self.term._uncoupledTerms + self.other._uncoupledTerms

    def _verifyVar(self, var):
        if var is not None:
            raise SolutionVariableNumberError('The solution variable should not be specified.')

        if len(self._vars) != len(self._uncoupledTerms):
            raise SolutionVariableNumberError

        return _AbstractBinaryTerm._verifyVar(self, _CoupledCellVariable(self._vars))

    @property
    def _buildExplcitIfOther(self):
        return False

    def _buildAndAddMatrices(self, var, SparseMatrix,  boundaryConditions=(), dt=None, transientGeomCoeff=None, diffusionGeomCoeff=None, buildExplicitIfOther=False):
        """Build matrices of constituent Terms and collect them

        Only called at top-level by `_prepareLinearSystem()`

        """

        from fipy.matrices.offsetSparseMatrix import OffsetSparseMatrix
        SparseMatrix =  OffsetSparseMatrix(SparseMatrix=SparseMatrix,
                                           numberOfVariables=len(self._vars),
                                           numberOfEquations=len(self._uncoupledTerms))
        matrix = SparseMatrix(mesh=var.mesh)
        RHSvectors = []

        for equationIndex, uncoupledTerm in enumerate(self._uncoupledTerms):

            SparseMatrix.equationIndex = equationIndex
            termRHSvector = 0
            termMatrix = SparseMatrix(mesh=var.mesh)

            for varIndex, tmpVar in enumerate(var.vars):

                SparseMatrix.varIndex = varIndex

                tmpVar, tmpMatrix, tmpRHSvector = uncoupledTerm._buildAndAddMatrices(tmpVar,
                                                                                     SparseMatrix,
                                                                                     boundaryConditions=(),
                                                                                     dt=dt,
                                                                                     transientGeomCoeff=uncoupledTerm._getTransientGeomCoeff(tmpVar),
                                                                                     diffusionGeomCoeff=uncoupledTerm._getDiffusionGeomCoeff(tmpVar),
                                                                                     buildExplicitIfOther=buildExplicitIfOther)

                termMatrix += tmpMatrix
                termRHSvector += tmpRHSvector

            uncoupledTerm._buildCache(termMatrix, termRHSvector)
            RHSvectors += [CellVariable(value=termRHSvector, mesh=var.mesh)]
            matrix += termMatrix

        return (var, matrix, _CoupledCellVariable(RHSvectors))

    def __repr__(self):
        return '(' + repr(self.term) + ' & ' + repr(self.other) + ')'

    def _getDefaultSolver(self, var, solver, *args, **kwargs):
        if solver and not solver._canSolveAsymmetric():
            import warnings
            warnings.warn("%s cannot solve asymmetric matrices" % solver)
        from fipy.solvers import DefaultAsymmetricSolver
        return solver or DefaultAsymmetricSolver(*args, **kwargs)

    def _calcVars(self):
        """
        This method returns the equations variables ordered by, transient terms,
        diffusion terms and other terms. Currently, this won't cure all of
        coupled equations ills, but it fixes the majority of issues, mainly with
        preconditioning. Some tests to make sure it works correctly.

        >>> from fipy import *
        >>> m = Grid1D(nx=2)
        >>> v0 = CellVariable(mesh=m, name='v0')
        >>> v1 = CellVariable(mesh=m, name='v1')
        >>> v2 = CellVariable(mesh=m, name='v2')
        >>> ConvectionTerm = CentralDifferenceConvectionTerm
        >>> eq0 = TransientTerm(var=v0) + VanLeerConvectionTerm(var=v0) + DiffusionTerm(var=v1)
        >>> eq1 = TransientTerm(var=v2) + ConvectionTerm(var=v2) + DiffusionTerm(var=v2) + ConvectionTerm(var=v1) + ImplicitSourceTerm(var=v1)
        >>> eq2 = ImplicitSourceTerm(var=v1) + 1 + ImplicitSourceTerm(var=v0) + 1 + DiffusionTerm(var=v1)
        >>> print((eq0 & eq1 & eq2)._vars)
        [v0, v2, v1]
        >>> print((eq0 & eq2 & eq1)._vars)
        [v0, v1, v2]
        >>> eq0 =  DiffusionTerm(var=v1) + TransientTerm(var=v0) + VanLeerConvectionTerm(var=v0)
        >>> print((eq0 & eq2 & eq1)._vars)
        [v0, v1, v2]
        >>> print((eq2 & eq0 & eq1)._vars)
        [v1, v0, v2]
        >>> print((eq2 & eq0 & eq1)([v1, v2, v0])._vars)
        [v1, v2, v0]
        >>> print((eq2 & eq0 & eq1)([v1, v2, v0, v2])._vars) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.
        >>> print((eq2 & eq0 & eq1)([v1, v2, 1])._vars) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Variable not in previously defined variables for this coupled equation.
        >>> print((eq2 & eq0 & eq1)([v1, v2, v1])._vars) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.
        >>> print((eq2 & eq0 & eq1)([v1, v2])._vars) # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
            ...
        SolutionVariableNumberError: Different number of solution variables and equations.

        """

        ## set() is used to force comparison by reference rather than value
        unorderedVars = _AbstractBinaryTerm._calcVars(self)
        uncoupledTerms = self._uncoupledTerms

        if len(unorderedVars) == len(uncoupledTerms):
            unorderedVars = set(unorderedVars)
            orderedVars = [None] * len(uncoupledTerms)

            for fnc in (lambda index, term: term._transientVars,
                        lambda index, term: term._diffusionVars,
                        lambda index, term: list(unorderedVars)):
                for index, term in enumerate(uncoupledTerms):
                    if orderedVars[index] is None:
                        _vars = fnc(index, term)
                        if  _vars != [] and _vars[0] in unorderedVars:
                            orderedVars[index] = _vars[0]
                            unorderedVars.remove(_vars[0])

            return orderedVars
        else:
            ## Constituent _CoupledBinaryTerms don't necessarily have the same
            ## number of equations and variables so ordering is unnecessary.
            return unorderedVars

    def __call__(self, _vars):
        _vars = list(_vars)

        if len(_vars) != len(self._vars) or len(set(_vars)) != len(self._vars):
            raise SolutionVariableNumberError

        for var in _vars:
            if var not in set(self._vars):
                raise SolutionVariableNumberError('Variable not in previously defined variables for this coupled equation.')

        self._internalVars = _vars
        return self

    def _test(self):
        """
        Offset tests

        >>> from fipy import *
        >>> m = Grid1D(nx=3)
        >>> v0 = CellVariable(mesh=m, value=0.)
        >>> v1 = CellVariable(mesh=m, value=1.)
        >>> eq0 = TransientTerm(var=v0) - DiffusionTerm(coeff=1., var=v0) - DiffusionTerm(coeff=2., var=v1)
        >>> eq1 = TransientTerm(var=v1) - DiffusionTerm(coeff=3., var=v0) - DiffusionTerm(coeff=4., var=v1)
        >>> eq = eq0 & eq1
        >>> var, matrix, RHSvector = eq._buildAndAddMatrices(var=eq._verifyVar(None), SparseMatrix=DefaultSolver()._matrixClass, dt=1.)
        >>> print(var.globalValue)
        [ 0.  0.  0.  1.  1.  1.]
        >>> print(RHSvector.globalValue)
        [ 0.  0.  0.  1.  1.  1.]
        >>> print(numerix.allequal(matrix.numpyArray,
        ...                        [[ 2, -1,  0,  2, -2,  0],
        ...                         [-1,  3, -1, -2,  4, -2],
        ...                         [ 0, -1,  2,  0, -2,  2],
        ...                         [ 3, -3,  0,  5, -4,  0],
        ...                         [-3,  6, -3, -4,  9, -4],
        ...                         [ 0, -3,  3,  0, -4,  5]]))
        True

        >>> m = Grid1D(nx=6)
        >>> v0 = CellVariable(mesh=m, value=0.)
        >>> v1 = CellVariable(mesh=m, value=1.)
        >>> eq0 = TransientTerm(var=v0) - DiffusionTerm(coeff=1., var=v0) - DiffusionTerm(coeff=2., var=v1)
        >>> eq1 = TransientTerm(var=v1) - DiffusionTerm(coeff=3., var=v0) - DiffusionTerm(coeff=4., var=v1)
        >>> eq = eq0 & eq1
        >>> var, matrix, RHSvector = eq._buildAndAddMatrices(var=eq._verifyVar(None), SparseMatrix=DefaultSolver()._matrixClass, dt=1.)
        >>> print(var.globalValue)
        [ 0.  0.  0.  0.  0.  0.  1.  1.  1.  1.  1.  1.]
        >>> print(RHSvector.globalValue)
        [ 0.  0.  0.  0.  0.  0.  1.  1.  1.  1.  1.  1.]
        >>> print(numerix.allequal(matrix.numpyArray,
        ...                        [[ 2, -1,  0,  0,  0,  0,  2, -2,  0,  0,  0,  0],
        ...                         [-1,  3, -1,  0,  0,  0, -2,  4, -2,  0,  0,  0],
        ...                         [ 0, -1,  3, -1,  0,  0,  0, -2,  4, -2,  0,  0],
        ...                         [ 0,  0, -1,  3, -1,  0,  0,  0, -2,  4, -2,  0],
        ...                         [ 0,  0,  0, -1,  3, -1,  0,  0,  0, -2,  4, -2],
        ...                         [ 0,  0,  0,  0, -1,  2,  0,  0,  0,  0, -2,  2],
        ...                         [ 3, -3,  0,  0,  0,  0,  5, -4,  0,  0,  0,  0],
        ...                         [-3,  6, -3,  0,  0,  0, -4,  9, -4,  0,  0,  0],
        ...                         [ 0, -3,  6, -3,  0,  0,  0, -4,  9, -4,  0,  0],
        ...                         [ 0,  0, -3,  6, -3,  0,  0,  0, -4,  9, -4,  0],
        ...                         [ 0,  0,  0, -3,  6, -3,  0,  0,  0, -4,  9, -4],
        ...                         [ 0,  0,  0,  0, -3,  3,  0,  0,  0,  0, -4,  5]]))
        True

        >>> m = Grid1D(nx=3)
        >>> v0 = CellVariable(mesh=m, value=0.)
        >>> v1 = CellVariable(mesh=m, value=1.)
        >>> diffTerm = DiffusionTerm(coeff=1., var=v0)
        >>> eq00 = TransientTerm(var=v0) - DiffusionTerm(coeff=1., var=v0)
        >>> eq0 = eq00 - DiffusionTerm(coeff=2., var=v1)
        >>> eq1 = TransientTerm(var=v1) - DiffusionTerm(coeff=3., var=v0) - DiffusionTerm(coeff=4., var=v1)
        >>> eq0.cacheMatrix()
        >>> diffTerm.cacheMatrix()
        >>> (eq0 & eq1).solve(dt=1.)
        >>> print(numerix.allequal(eq0.matrix.numpyArray,
        ...                        [[ 2, -1,  0,  2, -2,  0],
        ...                         [-1,  3, -1, -2,  4, -2],
        ...                         [ 0, -1,  2,  0, -2,  2],
        ...                         [ 0,  0,  0,  0,  0,  0],
        ...                         [ 0,  0,  0,  0,  0,  0],
        ...                         [ 0,  0,  0,  0,  0,  0]]))
        True
        >>> ## This correctly returns None because we lost the handle to the DiffusionTerm when it's negated.
        >>> print(diffTerm.matrix)
        None

        Check `diffusionGeomCoeff` is determined correctly (ticket:329)

        >>> m = Grid1D(nx=3)
        >>> v0 = CellVariable(mesh=m, value=0.)
        >>> v1 = CellVariable(mesh=m, value=1.)
        >>> eq0 = PowerLawConvectionTerm(coeff=100., var=v0) - DiffusionTerm(coeff=1., var=v0)
        >>> eq1 = PowerLawConvectionTerm(coeff=-0.001, var=v1) - DiffusionTerm(coeff=3., var=v1)
        >>> eq = eq0 & eq1
        >>> var, matrix, RHSvector = eq._buildAndAddMatrices(var=eq._verifyVar(None), SparseMatrix=DefaultSolver()._matrixClass)
        >>> print(var.globalValue)
        [ 0.  0.  0.  1.  1.  1.]
        >>> print(RHSvector.globalValue)
        [ 0.  0.  0.  0.  0.  0.]
        >>> print(numerix.allclose(matrix.numpyArray,
        ...                        [[ 100, 1e-15,      0,       0,       0,       0],
        ...                         [-100,   100,  1e-15,       0,       0,       0],
        ...                         [   0,  -100, -1e-15,       0,       0,       0],
        ...                         [   0,     0,      0,  2.9995, -3.0005,       0],
        ...                         [   0,     0,      0, -2.9995,  6.0000, -3.0005],
        ...                         [   0,     0,      0,       0, -2.9995,  3.0005]]))
        True
        """

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()




from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.viewers.vtkViewer.vtkCellViewer import VTKCellViewer
from fipy.viewers.vtkViewer.vtkFaceViewer import VTKFaceViewer

__all__ = ["VTKViewer"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]
__all__.extend(vtkCellViewer.__all__)
__all__.extend(vtkFaceViewer.__all__)

def VTKViewer(vars, title=None, limits={}, **kwlimits):
    """Generic function for creating a `VTKViewer`.

    The `VTKViewer` factory will search the module tree and return an
    instance of the first `VTKViewer` it finds of the correct dimension
    and rank.

    Parameters
    ----------
    vars : ~fipy.variables.cellVariable.CellVariable or ~fipy.variables.faceVariable.FaceVariable or list
        the `MeshVariable` objects to display.
    title : str, optional
        displayed at the top of the `Viewer` window
    limits : dict, optional
        a (deprecated) alternative to limit keyword arguments
    float xmin, xmax, ymin, ymax, zmin, zmax, datamin, datamax : float, optional
        displayed range of data. Any limit set to
        a (default) value of `None` will autoscale.
    """
    if type(vars) not in [type([]), type(())]:
        vars = [vars]

    kwlimits.update(limits)

    try:
        return VTKCellViewer(vars=vars, title=title, **kwlimits)
    except TypeError:
        return VTKFaceViewer(vars=vars, title=title, **kwlimits)



from __future__ import unicode_literals
from fipy.meshes.builders.grid1DBuilder import _NonuniformGrid1DBuilder
from fipy.meshes.builders.grid1DBuilder import _UniformGrid1DBuilder
from fipy.meshes.builders.grid1DBuilder import _Grid1DBuilder
from fipy.meshes.builders.grid2DBuilder import _NonuniformGrid2DBuilder
from fipy.meshes.builders.grid2DBuilder import _UniformGrid2DBuilder
from fipy.meshes.builders.grid2DBuilder import _Grid2DBuilder
from fipy.meshes.builders.grid3DBuilder import _NonuniformGrid3DBuilder
from fipy.meshes.builders.grid3DBuilder import _UniformGrid3DBuilder
from fipy.meshes.builders.grid3DBuilder import _Grid3DBuilder
from fipy.meshes.builders.periodicGrid1DBuilder import _PeriodicGrid1DBuilder



"""
Periodic 1D Mesh
"""
from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

from fipy.tools import numerix

from fipy.meshes.nonUniformGrid1D import NonUniformGrid1D
from fipy.meshes.builders import _PeriodicGrid1DBuilder
from fipy.meshes.topologies.gridTopology import _PeriodicGrid1DTopology

__all__ = ["PeriodicGrid1D"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class PeriodicGrid1D(NonUniformGrid1D):
    """

    Creates a Periodic grid mesh.

        >>> mesh = PeriodicGrid1D(dx = (1, 2, 3))

        >>> print(numerix.allclose(numerix.nonzero(mesh.exteriorFaces)[0],
        ...                        [3])) # doctest: +PROCESSOR_0
        True

        >>> print(numerix.allclose(mesh.faceCellIDs.filled(-999),
        ...                        [[2, 0, 1, 2],
        ...                         [0, 1, 2, -999]])) # doctest: +PROCESSOR_0
        True

        >>> print(numerix.allclose(mesh._cellDistances,
        ...                        [ 2., 1.5, 2.5, 1.5])) # doctest: +PROCESSOR_0
        True

        >>> print(numerix.allclose(mesh._cellToCellDistances,
        ...                        [[ 2.,   1.5,  2.5],
        ...                         [ 1.5,  2.5,  2. ]])) # doctest: +PROCESSOR_0
        True

        >>> print(numerix.allclose(mesh.faceNormals,
        ...                        [[ 1.,  1.,  1.,  1.]])) # doctest: +PROCESSOR_0
        True

        >>> print(numerix.allclose(mesh._cellVertexIDs,
        ...                        [[1, 2, 2],
        ...                        [0, 1, 0]])) # doctest: +PROCESSOR_0
        True
    """
    def __init__(self, dx = 1., nx = None, overlap=2, *args, **kwargs):

        super(PeriodicGrid1D, self).__init__(dx = dx, nx = nx, overlap=overlap,
                                             _BuilderClass=_PeriodicGrid1DBuilder,
                                             _TopologyClass=_PeriodicGrid1DTopology, *args, **kwargs)
        self._nonPeriodicCellFaceIDs = numerix.array(super(PeriodicGrid1D, self).cellFaceIDs)
        self._makePeriodic()

    def _makePeriodic(self):
        if self.occupiedNodes == 1:
            self._connectFaces(numerix.nonzero(self.facesLeft),
                               numerix.nonzero(self.facesRight))

    @property
    def cellCenters(self):
        """Defined outside of a geometry class since we need the `CellVariable`
        version of `cellCenters`; that is, the `cellCenters` defined in
        :mod:`fipy.meshes.mesh` and not in any geometry (since a `CellVariable` requires
        a reference to a mesh)."""
        return super(PeriodicGrid1D, self).cellCenters \
                % numerix.sum(self.globalNumberOfCells * self.args['dx'])

    def _translate(self, vector):
        """
        Test for ticket:298.

        >>> from fipy import *
        >>> m = PeriodicGrid1D(nx=2) + [[-1]]
        >>> print(CellVariable(mesh=m, value=m.cellCenters[0]))
        [-0.5  0.5]

        """
        newCoords = self.vertexCoords + vector
        newmesh = self.__class__(**self.args)
        from fipy.meshes.mesh1D import Mesh1D
        Mesh1D.__init__(newmesh, newCoords, numerix.array(self.faceVertexIDs), self._nonPeriodicCellFaceIDs, communicator=self.communicator)
        newmesh._makePeriodic()
        return newmesh

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()




from __future__ import unicode_literals
from fipy.steppers.stepper import Stepper

__all__ = ["PseudoRKQSStepper"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class PseudoRKQSStepper(Stepper):
    """
    Adaptive stepper based on the ``rkqs`` (Runge-Kutta
    "quality-controlled" stepper) algorithm of Numerical Recipes in C: 2nd
    Edition, Section 16.2.

    Not really appropriate, since we're not doing Runge-Kutta steps
    in the first place, but works OK.
    """
    def __init__(self, vardata=(), safety=0.9, pgrow=-0.2, pshrink=-0.25, errcon=1.89e-4):
        Stepper.__init__(self, vardata=vardata)
        self.safety = safety
        self.pgrow = pgrow
        self.pshrink = pshrink
        self.errcon = errcon

    def _step(self, dt, dtPrev, sweepFn, failFn, *args, **kwargs):
        residual = 1e100
        while residual > 1.:
            residual = sweepFn(vardata=self.vardata, dt=dt, *args, **kwargs)

            if residual > 1.:
                # step failed
                failFn(vardata=self.vardata, dt=dt, *args, **kwargs)

                # revert
                for var, eqn, bcs in self.vardata:
                    var.setValue(var.old)

                    dt = max(self.safety * dt * residual**self.pgrow, 0.1 * dt)

                dt = self._lowerBound(dt)

        if residual > self.errcon:
            dtNext = dt * self.safety * residual**self.pshrink
        else:
            dtNext = 5 * dt

        return dt, dtNext



from __future__ import division
from builtins import range
from past.utils import old_div
__docformat__ = 'restructuredtext'

import os

from petsc4py import PETSc

from fipy.solvers.petsc.petscSolver import PETScSolver

__all__ = ["LinearLUSolver"]

class LinearLUSolver(PETScSolver):

    """
    The `LinearLUSolver` is an interface to the LU preconditioner in PETSc.
    A direct solve is performed.

    """
      
    def __init__(self, tolerance=1e-10, iterations=10, precon="lu"):
        """
        :Parameters:
          - `tolerance`: The required error tolerance.
          - `iterations`: The maximum number of iterative steps to perform.
          - `precon`: *Ignored*. 

        """
        PETScSolver.__init__(self, tolerance=tolerance,
                             iterations=iterations, precon="lu")

    def _solve_(self, L, x, b):
        ksp = PETSc.KSP()
        ksp.create(PETSc.COMM_WORLD)
        ksp.setType("preonly")
        ksp.getPC().setType(self.preconditioner)
        # TODO: SuperLU invoked with PCFactorSetMatSolverType(pc, MATSOLVERSUPERLU)
        #       see: http://www.mcs.anl.gov/petsc/petsc-dev/src/ksp/ksp/examples/tutorials/ex52.c.html
        # PETSc.PC().setFactorSolverType("superlu")
        
        L.assemblyBegin()
        L.assemblyEnd()
        ksp.setOperators(L)
        ksp.setFromOptions()
        
        for iteration in range(self.iterations):
            errorVector = L * x - b
            tol = errorVector.norm()
            
            if iteration == 0:
                tol0 = tol
                
            if (tol / tol0) <= self.tolerance:
                break
                
            xError = x.copy()

            ksp.solve(errorVector, xError)
            x -= xError
            
        if 'FIPY_VERBOSE_SOLVER' in os.environ:
            from fipy.tools.debug import PRINT
#             L.view()
#             b.view()
            PRINT('solver:', ksp.type)
            PRINT('precon:', ksp.getPC().type)
            PRINT('iterations: %d / %d' % (iteration+1, self.iterations))
            PRINT('residual:', errorVector.norm(1))




from __future__ import unicode_literals
__all__ = []

from fipy.tests.doctestPlus import _LateImportDocTestSuite
import fipy.tests.testProgram
from fipy.solvers import solver

if solver == 'trilinos':
    docTestModuleNames = ('trilinosMatrix', 'pysparseMatrix')
elif solver == 'no-pysparse':
    docTestModuleNames = ('trilinosMatrix',)
elif solver == 'scipy' or solver == 'pyamg':
    docTestModuleNames = ('scipyMatrix',)
elif solver == 'pysparse':
    docTestModuleNames = ('pysparseMatrix',)
elif solver == 'pyamgx':
    docTestModuleNames = ('scipyMatrix',)
elif solver == 'petsc':
    docTestModuleNames = ('petscMatrix',)
else:
    raise ImportError('Unknown solver package %s' % solver)

def _suite():
    return _LateImportDocTestSuite(docTestModuleNames=docTestModuleNames, base=__name__)

if __name__ == '__main__':
    fipy.tests.testProgram.main(defaultTest='_suite')



from __future__ import unicode_literals
from fipy.variables.meshVariable import _MeshVariable
from fipy.tools import numerix

__all__ = ["FaceVariable"]
from future.utils import text_to_native_str
__all__ = [text_to_native_str(n) for n in __all__]

class FaceVariable(_MeshVariable):
    @property
    def _variableClass(self):
        return FaceVariable

    def _getShapeFromMesh(mesh):
        """
        Return the shape of this variable type, given a particular mesh.
        """
        return (mesh.numberOfFaces,)
    _getShapeFromMesh = staticmethod(_getShapeFromMesh)

    def _getArithmeticBaseClass(self, other = None):
        """
        Given `self` and `other`, return the desired base class for an operation
        result.
        """
        if other is None:
            return FaceVariable

        return _MeshVariable._getArithmeticBaseClass(self, other)

    def copy(self):
        return self._getArithmeticBaseClass()(mesh=self.mesh,
                                              name=self.name + "_copy",
                                              value=self.value)

    @property
    def globalValue(self):
        return self._getGlobalValue(self.mesh._localNonOverlappingFaceIDs,
                                    self.mesh._globalNonOverlappingFaceIDs)

    def setValue(self, value, unit = None, where = None):
        _MeshVariable.setValue(self, value=self._globalToLocalValue(value), unit=unit, where=where)

    @property
    def divergence(self):
        r"""the divergence of `self`, :math:`\vec{u}`,

        .. math:: \nabla\cdot\vec{u} \approx \frac{\sum_f (\vec{u}\cdot\hat{n})_f A_f}{V_P}

        Returns
        -------
        divergence : fipy.variables.cellVariable.CellVariable
            one rank lower than `self`

        Examples
        --------

        >>> from fipy.meshes import Grid2D
        >>> from fipy.variables.cellVariable import CellVariable
        >>> mesh = Grid2D(nx=3, ny=2)
        >>> from builtins import range
        >>> var = CellVariable(mesh=mesh, value=list(range(3*2)))
        >>> print(var.faceGrad.divergence)
        [ 4.  3.  2. -2. -3. -4.]

        """
        if not hasattr(self, '_divergence'):
            from fipy.variables.addOverFacesVariable import _AddOverFacesVariable

            s = (slice(0, None, None),) + (numerix.newaxis,) * (len(self.shape) - 2) + (slice(0, None, None),)
            self._divergence = _AddOverFacesVariable((self * self.mesh._orientedAreaProjections[s]).sum(0))

        return self._divergence

    @property
    def _globalNumberOfElements(self):
        return self.mesh.globalNumberOfFaces

    @property
    def _globalOverlappingIDs(self):
        return self.mesh._globalOverlappingFaceIDs

    @property
    def _localNonOverlappingIDs(self):
        return self.mesh._localNonOverlappingFaceIDs

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

import os

from fipy.terms.term import Term
from fipy.terms.explicitSourceTerm import _ExplicitSourceTerm
from fipy.terms import ExplicitVariableError

class _AbstractBinaryTerm(Term):
    def __init__(self, term, other):

        if not isinstance(other, Term):
            other = _ExplicitSourceTerm(coeff=other, var=term.var)

        self.term = term
        self.other = other

        if term.var is None:
            if other.var is None:
                pass
            else:
                raise ExplicitVariableError
        else:
            if other.var is None:
                if isinstance(other, _ExplicitSourceTerm):
                    other.var = term.var
                else:
                    raise ExplicitVariableError

        Term.__init__(self, var=self._vars[0])

    def _addNone(self, arg0, arg1):
        if arg0 is None and arg1 is None:
            return None
        elif arg0 is None:
            return arg1
        elif arg1 is None:
            return arg0
        else:
            return arg0 + arg1

    def __neg__(self):
        r"""
         Negate a `_BinaryTerm`.

           >>> -(__NonDiffusionTerm(coeff=1.) - __NonDiffusionTerm(coeff=2.))
           (__NonDiffusionTerm(coeff=-1.0) + __NonDiffusionTerm(coeff=2.0))

        """

        return (-self.term) + (-self.other)

    def _calcVars(self):
        """Collect (non-redundant) list of all `CellVariables`
        this binary term solves for.

        note: cannot use a set because its order can be different
        on different processors
        """
        ids = [id(v) for v in self.term._vars]
        othervars = [v for v in self.other._vars if id(v) not in ids]
        return self.term._vars + othervars

    @property
    def _vars(self):
        if not hasattr(self, '_internalVars'):
            self._internalVars = self._calcVars()
        return self._internalVars

    @property
    def _transientVars(self):
        return self.term._transientVars + self.other._transientVars

    @property
    def _diffusionVars(self):
        return self.term._diffusionVars + self.other._diffusionVars

    def _checkVar(self, var):
        self.term._checkVar(var)
        self.other._checkVar(var)


from fipy.terms.nonDiffusionTerm import _NonDiffusionTerm
class __NonDiffusionTerm(_NonDiffusionTerm):
    """
    Dummy subclass for tests
    """
    pass

def _test():
    import fipy.tests.doctestPlus
    return fipy.tests.doctestPlus.testmod()

if __name__ == "__main__":
    _test()



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

from fipy.meshes.builders.abstractGridBuilder import _AbstractGridBuilder
from fipy.tools import numerix
from fipy.tools import vector
from fipy.tools.dimensions.physicalField import PhysicalField
from fipy.meshes.builders.utilityClasses import (_UniformNumPts,
                                                 _DOffsets,
                                                 _UniformOrigin,
                                                 _NonuniformNumPts)

class _Grid3DBuilder(_AbstractGridBuilder):

    def buildGridData(self, *args, **kwargs):
        super(_Grid3DBuilder, self).buildGridData(*args, **kwargs)

        self.numberOfHorizontalRows = self.spatialDict["numHorizontalRows"]
        self.numberOfVerticalColumns = self.spatialDict["numVerticalCols"]
        self.numberOfLayersDeep = self.spatialDict["numLayersDeep"]

    def _calcShape(self):
        return (self.ns[0], self.ns[1], self.ns[2])

    def _calcPhysicalShape(self):
        """Return physical dimensions of `Grid3D`
        """
        from fipy.tools.dimensions.physicalField import PhysicalField
        return PhysicalField(value = (self.ns[0] * self.ds[0] * self.scale,
                                      self.ns[1] * self.ds[1] * self.scale,
                                      self.ns[2] * self.ds[2] * self.scale))

    def _calcMeshSpacing(self):
        return numerix.array((self.ds[0], self.ds[1], self.ds[2]))[..., numerix.newaxis]

    @property
    def _specificGridData(self):
        return [self.numberOfXYFaces,
                self.numberOfXZFaces,
                self.numberOfYZFaces,
                self.numberOfHorizontalRows,
                self.numberOfVerticalColumns,
                self.numberOfLayersDeep]


    @staticmethod
    def createVertices(dx, dy, dz, nx, ny, nz,
                       numVertices, numHorizRows, numVertCols):
        x = _AbstractGridBuilder.calcVertexCoordinates(dx, nx)
        x = numerix.resize(x, (numVertices,))

        y = _AbstractGridBuilder.calcVertexCoordinates(dy, ny)
        y = numerix.repeat(y, numVertCols)
        y = numerix.resize(y, (numVertices,))

        z = _AbstractGridBuilder.calcVertexCoordinates(dz, nz)
        z = numerix.repeat(z, numHorizRows * numVertCols)
        z = numerix.resize(z, (numVertices,))

        return numerix.array((x, y, z))

    @staticmethod
    def createFaces(nx, ny, nz):
        """
        XY faces are first, then XZ faces, then YZ faces
        """
        ## do the XY faces
        v1 = numerix.arange((nx + 1) * (ny))
        v1 = vector.prune(v1, nx + 1, nx)
        v1 = _Grid3DBuilder._repeatWithOffset(v1, (nx + 1) * (ny + 1), nz + 1)
        v2 = v1 + 1
        v3 = v1 + (nx + 2)
        v4 = v1 + (nx + 1)
        XYFaces = numerix.array((v1, v2, v3, v4))

        ## do the XZ faces
        v1 = numerix.arange((nx + 1) * (ny + 1))
        v1 = vector.prune(v1, nx + 1, nx)
        v1 = _Grid3DBuilder._repeatWithOffset(v1, (nx + 1) * (ny + 1), nz)
        v2 = v1 + 1
        v3 = v1 + ((nx + 1)*(ny + 1)) + 1
        v4 = v1 + ((nx + 1)*(ny + 1))
        XZFaces = numerix.array((v1, v2, v3, v4))

        ## do the YZ faces
        v1 = numerix.arange((nx + 1) * ny)
        v1 = _Grid3DBuilder._repeatWithOffset(v1, (nx + 1) * (ny + 1), nz)
        v2 = v1 + (nx + 1)
        v3 = v1 + ((nx + 1)*(ny + 1)) + (nx + 1)
        v4 = v1 + ((nx + 1)*(ny + 1))
        YZFaces = numerix.array((v1, v2, v3, v4))

        numberOfXYFaces = (nx * ny * (nz + 1))
        numberOfXZFaces = (nx * (ny + 1) * nz)
        numberOfYZFaces = ((nx + 1) * ny * nz)
        numberOfFaces = numberOfXYFaces + numberOfXZFaces + numberOfYZFaces

        return ([numberOfXYFaces, numberOfXZFaces, numberOfYZFaces, numberOfFaces],
                numerix.concatenate((XYFaces, XZFaces, YZFaces), axis=1))

    @staticmethod
    def createCells(nx, ny, nz, numXYFaces, numXZFaces, numYZFaces):
        """
        cells = (front face, back face, left face, right face, bottom face, top face)
        front and back faces are YZ faces
        left and right faces are XZ faces
        top and bottom faces are XY faces
        """
        ## front and back faces
        frontFaces = numerix.arange(numYZFaces)
        frontFaces = vector.prune(frontFaces, nx + 1, nx)
        frontFaces = frontFaces + numXYFaces + numXZFaces
        backFaces = frontFaces + 1

        ## left and right faces
        leftFaces = numerix.arange(nx * ny)
        leftFaces = _Grid3DBuilder._repeatWithOffset(leftFaces, nx * (ny + 1), nz)
        leftFaces = numerix.ravel(leftFaces)
        leftFaces = leftFaces + numXYFaces
        rightFaces = leftFaces + nx

        ## bottom and top faces
        bottomFaces = numerix.arange(nx * ny * nz)
        topFaces = bottomFaces + (nx * ny)

        return numerix.array((frontFaces, backFaces, leftFaces,
                              rightFaces, bottomFaces, topFaces))

    @staticmethod
    def _repeatWithOffset(array, offset, reps):
        a = numerix.fromfunction(lambda rnum, x: array + (offset * rnum),
                                 (reps, numerix.size(array))).astype('l')
        return numerix.ravel(a)


    def _packOverlap(self, first, second):
        return {'left': 0, 'right': 0, 'bottom' : 0, 'top' : 0,
                'front': first, 'back': second}

    def _packOffset(self, arg):
        return (0, 0, arg)

class _NonuniformGrid3DBuilder(_Grid3DBuilder):

    def __init__(self):
        self.NumPtsCalcClass = _NonuniformNumPts

        super(_NonuniformGrid3DBuilder, self).__init__()

    def buildGridData(self, *args, **kwargs):
        super(_NonuniformGrid3DBuilder, self).buildGridData(*args, **kwargs)

        ([self.Xoffset, self.Yoffset, self.Zoffset],
         self.ds) = _DOffsets.calcDOffsets(self.ds,
                                           self.ns,
                                           self.offset)

        self.vertices = _Grid3DBuilder.createVertices(self.ds[0], self.ds[1],
                                                      self.ds[2],
                                                      self.ns[0], self.ns[1],
                                                      self.ns[2],
                                                      self.numberOfVertices,
                                                      self.numberOfHorizontalRows,
                                                      self.numberOfVerticalColumns) \
                         + ((self.Xoffset,), (self.Yoffset,), (self.Zoffset,))

        numFacesList, self.faces = _Grid3DBuilder.createFaces(self.ns[0],
                                                              self.ns[1],
                                                              self.ns[2])

        self.numberOfXYFaces = numFacesList[0]
        self.numberOfXZFaces = numFacesList[1]
        self.numberOfYZFaces = numFacesList[2]
        self.numberOfFaces   = numFacesList[3]

        self.cells = _Grid3DBuilder.createCells(self.ns[0],
                                                self.ns[1],
                                                self.ns[2],
                                                self.numberOfXYFaces,
                                                self.numberOfXZFaces,
                                                self.numberOfYZFaces)

    @property
    def _specificGridData(self):
        return super(_NonuniformGrid3DBuilder, self)._specificGridData \
                + [self.vertices,
                   self.faces,
                   self.cells,
                   self.Xoffset, self.Yoffset, self.Zoffset]

class _UniformGrid3DBuilder(_Grid3DBuilder):

    def __init__(self):
        self.NumPtsCalcClass = _UniformNumPts

        super(_UniformGrid3DBuilder, self).__init__()

    def buildGridData(self, ds, ns, overlap, communicator, origin):
        super(_UniformGrid3DBuilder, self).buildGridData(ds, ns, overlap,
                                                        communicator)

        self.origin = _UniformOrigin.calcOrigin(origin,
                                                self.offset, self.ds, self.scale)

        self.numberOfXYFaces = self.ns[0] * self.ns[1] * (self.ns[2] + 1)
        self.numberOfXZFaces = self.ns[0] * (self.ns[1] + 1) * self.ns[2]
        self.numberOfYZFaces = (self.ns[0] + 1) * self.ns[1] * self.ns[2]
        self.numberOfFaces = self.numberOfXYFaces + self.numberOfXZFaces \
                              + self.numberOfYZFaces

    @property
    def _specificGridData(self):
        return super(_UniformGrid3DBuilder, self)._specificGridData \
                + [self.origin]



from __future__ import unicode_literals
__docformat__ = 'restructuredtext'

__all__ = []

from fipy.meshes.builders.grid1DBuilder import _NonuniformGrid1DBuilder

class _PeriodicGrid1DBuilder(_NonuniformGrid1DBuilder):

    def buildGridData(self, *args, **kwargs):
        kwargs["cacheOccupiedNodes"] = True
        return super(_PeriodicGrid1DBuilder, self).buildGridData(*args,
                                                                **kwargs)

    def _buildOverlap(self, overlap, procID, occupiedNodes):
        if occupiedNodes == 1:
            return super(_PeriodicGrid1DBuilder, self)._buildOverlap(overlap,
                     procID, occupiedNodes)
        else:
            return (overlap, overlap, {'left': overlap, 'right': overlap})
